wp.media.view.Settings.Waveplayer = wp.media.view.Settings.extend({
	className: 'waveplayer-settings',
	template:  wp.template('waveplayer-settings'),
    
});


var $ = jQuery;

$.ajax({
    url: waveplayer_ajax.ajax_url,
    data: { action: 'waveplayer_load_options' },
    dataType: 'json',
    type: 'post',
    success: function( response ) {

    },
    error: function( response ) {
        
    }
});

wp.media.waveplayer = new wp.media.collection({
	tag: 'waveplayer',
	editTitle : wp.media.view.l10n.editWaveplayerTitle,
	defaults : {
		id: wp.media.view.settings.post.id,
		wave_color:             '#555',
		wave_color_2:       '#eee',
		progress_color:         '#048',
		progress_color_2:   '#08e',
		cursor_color:           'transparent',
		wave_mode:              '3',
		size:                   'md',
		style:                  'light',
		shape:                  'circle',
		autoplay:               false,
        repeat_all:             false,
        shuffle:                false,
	},
});

// supersede the default MediaFrame.Post view
var postMediaFrame = wp.media.view.MediaFrame.Post;
wp.media.view.MediaFrame.Post = postMediaFrame.extend({

    initialize: function() {
        var options = this.options;
        postMediaFrame.prototype.initialize.apply( this, arguments );
        
        this.states.add([
			new wp.media.controller.Library({
				id:         'waveplayer',
				title:      wp.media.view.l10n.createWaveplayerTitle,
				priority:   60,
				toolbar:    'main-waveplayer',
				filterable: 'uploaded',
				multiple:   'add',
				editable:   false,

				library:  wp.media.query( _.defaults({
					type: 'audio'
				}, options.library ) )
			}),
            
			// Waveplayer states.
			new wp.media.controller.CollectionEdit({
				type: 'audio',
				collectionType: 'waveplayer',
				title:          wp.media.view.l10n.editWaveplayerTitle,
				SettingsView:   wp.media.view.Settings.Waveplayer,
				library:        options.selection,
				editing:        options.editing,
				menu:           'waveplayer',
				dragInfoText:   wp.media.view.l10n.waveplayerDragInfo,
				dragInfo:       false
			}),

			new wp.media.controller.CollectionAdd({
				type: 'audio',
				collectionType: 'waveplayer',
				title: wp.media.view.l10n.addToWaveplayerTitle
			}),

        ]);
	},

	bindHandlers: function() {
		var handlers, checkCounts;

		postMediaFrame.prototype.bindHandlers.apply( this, arguments );

		this.on( 'activate', this.activate, this );

		// Only bother checking media type counts if one of the counts is zero
		checkCounts = _.find( this.counts, function( type ) {
			return type.count === 0;
		} );

		if ( typeof checkCounts !== 'undefined' ) {
			this.listenTo( wp.media.model.Attachments.all, 'change:type', this.mediaTypeCounts );
		}

		this.on( 'menu:create:waveplayer', this.createMenu, this );
		this.on( 'toolbar:create:main-waveplayer', this.createToolbar, this );
        this.on( 'update', this.update, this);

		handlers = {
			menu: {
				'waveplayer': 'waveplayerMenu',
			},

			toolbar: {
				'main-waveplayer':	'mainWaveplayerToolbar',
				'waveplayer-edit':	'waveplayerEditToolbar',
				'waveplayer-add':	'waveplayerAddToolbar',
			}
		};

		_.each( handlers, function( regionHandlers, region ) {
			_.each( regionHandlers, function( callback, handler ) {
				this.on( region + ':render:' + handler, this[ callback ], this );
			}, this );
		}, this );
    },
    
	waveplayerMenu: function( view ) {
		var lastState = this.lastState(),
			previous = lastState && lastState.id,
			frame = this;
            
		view.set({
			cancel: {
				text:     wp.media.view.l10n.cancelWaveplayerTitle,
				priority: 20,
				click:    function() {
					if ( previous ) {
						frame.setState( previous );
					} else {
						frame.close();
					}
				}
			},
			separateCancel: new wp.media.View({
				className: 'separator',
				priority: 40
			})
		});

	},
    
	mainWaveplayerToolbar: function( view ) {
		var controller = this;

		this.selectionStatusToolbar( view );

		view.set( 'waveplayer', {
			style:    'primary',
			text:     wp.media.view.l10n.createNewWaveplayer,
			priority: 100,
			requires: { selection: true },

			click: function() {
				var selection = controller.state().get('selection'),
					edit = controller.state('waveplayer-edit'),
					models = selection.where({ type: 'audio' });

				edit.set( 'library', new wp.media.model.Selection( models, {
					props:    selection.props.toJSON(),
					multiple: true
				}) );
                
				this.controller.setState('waveplayer-edit');

				// Keep focus inside media modal
				// after jumping to playlist view
				this.controller.modal.focusManager.focus();
			}
		});
	},
    
	waveplayerEditToolbar: function() {
		var editing = this.state().get('editing');
		this.toolbar.set( new wp.media.view.Toolbar({
			controller: this,
			items: {
				insert: {
					style:    'primary',
					text:     editing ? wp.media.view.l10n.updateWaveplayer : wp.media.view.l10n.insertWaveplayer,
					priority: 80,
					requires: { library: true },

					/**
					 * @fires wp.media.controller.State#update
					 */
					click: function() {
						var controller = this.controller,
							state = controller.state();
                        
						controller.close();
                        wp.media.editor.insert( wp.media.waveplayer.shortcode(state.get('library')).string() );

						// Restore and reset the default state.
						controller.setState( controller.options.state );
						controller.reset();
					}
				}
			}
		}) );
	},

	waveplayerAddToolbar: function() {
		this.toolbar.set( new wp.media.view.Toolbar({
			controller: this,
			items: {
				insert: {
					style:    'primary',
					text:     wp.media.view.l10n.addToWaveplayer,
					priority: 80,
					requires: { selection: true },

					/**
					 * @fires wp.media.controller.State#reset
					 */
					click: function() {
						var controller = this.controller,
							state = controller.state(),
							edit = controller.state('waveplayer-edit');

						edit.get('library').add( state.get('selection').models );
						state.trigger('reset');
						controller.setState('waveplayer-edit');
					}
				}
			}
		}) );
	},
    
    activate: function() {
//        jQuery('.waveplayer-color-picker').wpColorPicker();
    }
    
});

var wvpl = {
    action: 'parse-waveplayer-shortcode',
    
    state: ['waveplayer-edit'],
    
    initialize: function() {
        var self = this,
            media = wp.media;
            
        wp.ajax.post( this.action, {
            post_ID: media.view.settings.post.id,
            type: this.shortcode.tag,
            shortcode: this.shortcode.string()
        } )
        .done( function( response ) {
            self.render( response );
        } )
        .fail( function( response ) {
            if ( self.url ) {
                self.ignore = true;
                self.removeMarkers();
            } else {
                self.setError( response.message || response.statusText, 'admin-media' );
            }
        } );

        this.getEditors( function( editor ) {
            editor.on( 'wpview-selected', function() {
                self.pausePlayers();
            } );
        } );
    },

    pausePlayers: function() {
        var $ = jQuery;

        this.getNodes( function( editor, node, content ) {
            var win = $( 'iframe.wpview-sandbox', content ).get( 0 );

            if ( win && ( win = win.contentWindow ) && win.mejs ) {
                _.each( win.mejs.players, function( player ) {
                    try {
                        player.pause();
                    } catch ( e ) {}
                } );
            }
        } );
    },

    edit: function( text, update ) {
        var frame = wp.media['waveplayer'].edit(text);

        this.pausePlayers && this.pausePlayers();

        frame.on( 'update', function( selection ) {
            update( wp.media.waveplayer.shortcode( selection ).string(), type === 'waveplayer' );
        });

        frame.on( 'close', function() {
            frame.detach();
        } );

        frame.open();
//        jQuery('.waveplayer-color-picker').wpColorPicker();
    }
};
wp.mce.views.register( 'waveplayer', wvpl);

wvplAudio = {
    action: 'parse-waveplayer-single-shortcode',
    
    state: [ 'audio-details' ],

    initialize: function() {
        var self = this,
            media = wp.media;

        if ( this.url ) {
            this.loader = false;
            this.shortcode = media.embed.shortcode( {
                url: this.text
            } );
        }

        wp.ajax.post( this.action, {
            post_ID: media.view.settings.post.id,
            type: this.shortcode.tag,
            shortcode: this.shortcode.string()
        } )
        .done( function( response ) {
            self.render( response );
        } )
        .fail( function( response ) {
            if ( self.url ) {
                self.ignore = true;
                self.removeMarkers();
            } else {
                self.setError( response.message || response.statusText, 'admin-media' );
            }
        } );

        this.getEditors( function( editor ) {
            editor.on( 'wpview-selected', function() {
                self.pausePlayers();
            } );
        } );
    },

    edit: function( text, update ) {
        var frame = wp.media[ 'audio' ].edit( text );

        this.pausePlayers && this.pausePlayers();

        frame.on( 'update', function( selection ) {
            update( wp.media.audio.shortcode( selection ).string(), type === 'audio' );
        } );

        frame.on( 'close', function() {
            frame.detach();
        } );

        frame.open();
    },
    
    pausePlayers: function() {
        var $ = jQuery;
        
        this.getNodes( function( editor, node, content ) {
            var win = $( 'iframe.wpview-sandbox', content ).get( 0 );

            if ( win && ( win = win.contentWindow ) && win.mejs ) {
                _.each( win.mejs.players, function( player ) {
                    try {
                        player.pause();
                    } catch ( e ) {}
                } );
            }
        } );
    }
};
wp.mce.views.unregister( 'audio' );
wp.mce.views.register( 'audio', wvplAudio );

(function ($) {

    $('.waveplayer_tab').click(function(){
        $('.waveplayer_tab').removeClass('nav-tab-active');
        $(this).addClass('nav-tab-active');
        $('.waveplayer-option-page:visible').hide();
        $('#'+$(this).attr('id').replace('-tab','')).show();
        $('#waveplayer_current_tab').val($(this).attr('id').replace('-tab',''));
    });
    
    $('.waveplayer-color-picker').wpColorPicker({
        change: function(event, ui){
            $(event.target).val(ui.color.toString());
            $(event.target).attr('name') == 'background_color' ?
                changeBackground(ui.color.toString()) :
                redraw();
        }
    });
    
    $('select[name=waveplayer_wave_mode]').change(function() {
        $(this).val() > 0 ?
            $('#waveplayer_gap_width_group').removeClass('wvpl-inactive') :
            $('#waveplayer_gap_width_group').addClass('wvpl-inactive');
    });
    
    var changeBackground = function(color) {
        $('#wvpl-sample-waveform').css('background-color', color);
    }
    
    $('.wvpl_delete_peaks').click(function(){
        var el = $(this);
        $.ajax({
            url: waveplayer_ajax.ajax_url,
            type: 'post',
            data: [ { name: 'action', value: 'waveplayer_delete_peaks' }, { name: 'mode', value: $(this).attr('data-elements') } ],
            dataType: 'json',
            success: function(result) {
                var status = result.success ? 'updated' : 'error';
                el.parent().append($('<div/>', {class: status}).html($('<p/>').html(result.data.message)));
            }
        });
    });
    
    if ( $('#wvpl-sample-waveform').length ) {
    
        var sampleWaveform = Object.create(WaveSurfer);
            
        sampleWaveform.init({
            container:          '#wvpl-sample-waveform',
            waveColor:          $('[name=waveplayer_wave_color]').val(),
            waveColor2:         $('[name=waveplayer_wave_color_2]').val(),
            progressColor:      $('[name=waveplayer_progress_color]').val(),
            progressColor2:     $('[name=waveplayer_progress_color_2]').val(),
            cursorColor:        $('[name=waveplayer_cursor_color]').val(),
            cursorColor2:       $('[name=waveplayer_cursor_color_2]').val(),
            cursorWidth:        $('[name=waveplayer_cursor_width]').val(),
            hoverOpacity:       parseInt($('[name=waveplayer_hover_opacity]').val()),
            waveMode:           parseInt($('[name=waveplayer_wave_mode]').val()),
            gapWidth:           parseInt($('[name=waveplayer_gap_width]').val()),
            compression:        parseInt($('[name=waveplayer_wave_compression]').val()),
            asymmetry:          parseInt($('[name=waveplayer_wave_asymmetry]').val()),
            height:             $('#wvpl-sample-waveform').height() - parseInt($('#wvpl-sample-waveform').css("marginTop")) + parseInt($('#wvpl-sample-waveform').css("marginBottom")),
            normalize:          true,
            reflection:         true,
            hideScrollbar:      true,
        });
            
        var peaks;
        var loadPeaks = function() { 
            var waveform = $('.wvpl-waveform');
            
            sampleWaveform.drawer.playing = true;
            
            $.ajax({
                url: waveplayer_ajax.ajax_url,
                type: 'post',
                data: [ {name: 'action', value: 'waveplayer_read_peaks'}, {name: 'id', value: 'sample'} ],
                dataType: 'json',
                success: function(result) {
                    if (result.success) {
                        peaks = result.data.peaks;
                        redraw();
                    }
                },
            });
        }
        
        var redraw = function() {
            var params = sampleWaveform.params;
            
            params.waveColor =          $('[name=waveplayer_wave_color]').val();
            params.waveColor2 =         $('[name=waveplayer_wave_color_2]').val();
            params.progressColor =      $('[name=waveplayer_progress_color]').val();
            params.progressColor2 =     $('[name=waveplayer_progress_color_2]').val();
            params.cursorColor =        $('[name=waveplayer_cursor_color]').val();
            params.cursorColor2 =       $('[name=waveplayer_cursor_color_2]').val();
            params.cursorWidth =        $('[name=waveplayer_cursor_width]').val();
            params.hoverOpacity =       parseInt($('[name=waveplayer_hover_opacity]').val());
            params.waveMode =           parseInt($('[name=waveplayer_wave_mode]').val());
            params.gapWidth =           parseInt($('[name=waveplayer_gap_width]').val());
            params.compression =        parseInt($('[name=waveplayer_wave_compression]').val());
            params.asymmetry =          parseInt($('[name=waveplayer_wave_asymmetry]').val());
            
            sampleWaveform.drawer.clearWave();
            sampleWaveform.drawer.drawPeaks(peaks, $('#wvpl-sample-waveform').width() * sampleWaveform.params.pixelRatio);
            
            sampleWaveform.drawer.progress = 0.333;
            sampleWaveform.drawer.updatePosition(0.667);
            
        }
        
        $('input[name^="waveplayer"], select[name^="waveplayer"]').change(function() {
            redraw();
        });
        
        loadPeaks();
    }
    
})(jQuery);
