﻿/**
* waveplayer.js
*/

(function($){
    
    var WavePlayer = function(element) {
        
        var elem = $(element);
        var obj = this;
        
        var INTERVAL  = 200,
            ANIMATION = 250;
        
        var status = 'empty',
            n = -1;
            
        var wavesurfer = Object.create(WaveSurfer),
            peaks,
            timerHook,
            options,
            tracks;
            
        var repeatAll = false,
            autoplay = false;
            
        var audio               = elem.find('audio'),
            infoBar             = elem.find('.wvpl-infobar'),
            interface           = elem.find('.wvpl-interface'),
            posterImg           = elem.find('.wvpl-poster'),
            rightBox            = elem.find('.wvpl-right-box'),
            infoText            = elem.find('.wvpl-playing-info'),
            waveform            = elem.find('.wvpl-waveform'),
            nextBtn             = elem.find('.wvpl-next'),
            prevBtn             = elem.find('.wvpl-prev'),
            playBtn             = elem.find('.wvpl-play'),
            timeLine            = elem.find('.wvpl-timeline'),
            centerLine          = elem.find('.wvpl-centerline'),
            pointer             = elem.find('.wvpl-pointer'),
            playBtn             = elem.find('.wvpl-play'),
            infoBtn             = elem.find('.wvpl-info'),
            volumeBtn           = elem.find('.wvpl-volume');
                        
        var getData = function(name) {
            return elem.attr('data-'+name);
        }
        
        /**
        *
        * Initializes the wavesurfer object to the waveform element of the current WavePlayer instance
        *
        */
        var wavesurferInit = function() {
            
            wavesurfer.init({
                container:          waveform[0],
                waveColor:          getData('wave_color'),
                waveColor2:         getData('wave_color_2'),
                progressColor:      getData('progress_color'),
                progressColor2:     getData('progress_color_2'),
                cursorColor:        getData('cursor_color'),
                cursorColor2:       getData('cursor_color_2'),
                cursorWidth:        getData('cursor_width'),
                hoverOpacity:       parseInt(getData('hover_opacity')),
                waveMode:           parseInt(getData('wave_mode')),
                gapWidth:           parseInt(getData('gap_width')),
                compression:        parseInt(getData('wave_compression')),
                asymmetry:          parseInt(getData('wave_asymmetry')),
                height:             waveform.outerHeight(false),
                normalize:          true,
                reflection:         true,
                hideScrollbar:      true,
            });
                        
            /**
            *
            * Called when the wavesurfer object finishes reading the audio
            * tries to save a peak file in the peak subfolder of the plugin
            *
            */
            wavesurfer.on('ready', function() {

                peaks = wavesurfer.backend.getPeaks(960);
                
                if (tracks[n]['index']) {
                    
                    var postData = [
                        {name: 'action', value: 'waveplayer_write_peaks'},
                        {name: 'peaks', value: peaks},
                        {name: 'id', value: tracks[n]['index']}
                    ];
                    $.ajax({
                        url: waveplayer_ajax.ajax_url,
                        type: 'post',
                        data: postData,
                        dataType: 'json',
                        success: function(result) {
                            
                        },
                        error: function() {
                            
                        }
                    });
                }
            });
            
            /**
            *
            * Relocate the progress position when the user click on the waveform
            *
            */
            wavesurfer.drawer.on('click', function(event) {
                if (status == 'play') {
                    progress(event.offsetX / wavesurfer.container.clientWidth);
                    audio[0].currentTime = audio[0].duration * event.offsetX / wavesurfer.container.clientWidth;
                } else {
                    obj.play();
                }
            });
        }
        
        /**
        *
        * Reset the waveform element to display a blank canvas
        *
        */
        var progress = function(pos){
            wavesurfer.drawer.updateProgress(pos);
        }
        
        /**
        *
        * Reset the waveform element to display a blank canvas
        *
        */
        var waveReset = function(){
            wavesurfer.drawer.clearWave();
            progress(0);
        }
        
        /**
        *
        * Redraw the current waveform
        *
        */
        var waveRedraw = function(){
            if (n >= 0) {
                wavesurfer.params.height = waveform.height() - parseInt(waveform.css("marginTop")) + parseInt(waveform.css("marginBottom"));
                wavesurfer.updateDrawer();
                wavesurfer.drawer.drawPeaks(peaks, waveform.width() * wavesurfer.params.pixelRatio);
            }
        }

        /**
        *
        * Initializes the current WavePlayer instance
        *
        * @param params, not in use (for future development)
        */
        this.init = function(params) {
            
            if (getData('ids') == '' && getData('mp3') == '') {
                
                // if neither 'ids' nor 'mp3' were provided, hides the instance
                elem.hide();
                console.log(getData('ids'));
            } else {
        
                // load the WavePlayer settings from WordPress and sets the 'options' variable 
                $.ajax({
                    url: waveplayer_ajax.ajax_url,
                    type: 'post',
                    data: [
                        {name: 'action', value: 'waveplayer_load_options'},
                    ],
                    dataType: 'json',
                    success: function(result) {
                        options = result.data.options;
                        
                        // Initializes the wavesurfer plugin for this instance
                        wavesurferInit();
                        
                        // Gets the options for this player from the waveplayer dataset
                        autoplay = getData('autoplay');
                        repeatAll = getData('repeat_all');
                        
                        // Gets the playlist of tracks from the waveplayer dataset
                        loadPlaylist();
                        elem.animate({opacity: 1}, ANIMATION);
                    },
                    error: function() {
                        
                    }
                });
            }
        }
        
        /**
        *
        * Updates the current position of the player
        *
        */
        var timeUpdate = function() {
            var pc = audio[0].currentTime / audio[0].duration;
            progress(pc);
            elem.find('.wvpl-timeline .wvpl-position').css('width', pc + '%');
            var data = {
                position:   audio[0].currentTime,
                title:      tracks[n]['title']
            };
            elem.trigger('timeUpdate', data);
        }
        
        /**
        *
        * Returns the current position of the player
        *
        */
        this.currentTime = function() {
            return audio[0].currentTime;
        }
        
        /**
        *
        * Starts the playback
        *
        */
        this.play = function() {
            $('.waveplayer').each(function(index, el){
                if ( index != $('.waveplayer').index(elem) ) $(el).data('waveplayer').stop();
            });
            audio[0].play();
            playBtn.find('.wvpl-icon').removeClass('fa-play').addClass('fa-pause');
            playBtn.find('.wvpl-button').addClass('pause');
            status = 'play';
            wavesurfer.drawer.playing = true;
        }
        
        /**
        *
        * Pauses the playback
        *
        */
        this.pause = function() {
            audio[0].pause();
            playBtn.find('.wvpl-icon').removeClass('fa-pause').addClass('fa-play');
            playBtn.find('.wvpl-button').removeClass('pause');
            wavesurfer.drawer.playing = false;
            status = 'pause';
        }
        
        /**
         *
         * Stops the playback
         *
         * @param   (bool) reset    if true resets the playlist to the first track   
         *
         */
        this.stop = function(reset) {
            this.pause();
            progress(0);
            status = 'stop';
            if (reset) {
                n = -1;
                waveReset();
                obj.preloadTrack();
            }
        }
        
        /**
        *
        * Skips to the next or previous track
        *
        * @param    (bool) next     if true or undefined, skips to next track, if false, skip to previous track
        */
        this.skip = function(next) {
            audio[0].pause();
            posterImg.animate({opacity: 0},ANIMATION);
            rightBox.animate(
                {opacity: 0},
                ANIMATION,
                function() {
                    waveReset();
                    if (typeof(next) == 'undefined') next = true;
                    next ? n++ : n--;
                    if ( !repeatAll && ( (n >= tracks.length) || (n < 0) ) ) {
                        obj.stop(true);
                    } else {
                        if (n == tracks.length) n = 0;
                        if (n < 0) n = tracks.length - 1;
                        status = 'stop';
                        wavesurfer.drawer.playing = false;
                        clearInterval(timerHook);
                        obj.preloadTrack(true);
                    }
                }
            );
        }

        /**
        *
        * Preloads the current track
        *
        * @param playnow, if true, after preloading the track, automatically starts the playback
        */
        this.preloadTrack = function(playnow) {
            if (n < 0) n = 0;
            checkSkipState();
            
            if (getData('ids')) {
                
                $.ajax({
                    url: waveplayer_ajax.ajax_url,
                    type: 'post',
                    data: [ {name: 'action', value: 'waveplayer_read_peaks'}, {name: 'id', value: tracks[n]['index']} ],
                    dataType: 'json',
                    success: function(result) {
                        infoText.html( displayInfo( tracks[n] ) );
                        audio[0].src = tracks[n]['file'];
                        audio[0].load();
                        if (result.success) {
                            peaks = result.data.peaks;
                            wavesurfer.drawer.clearWave();
                            wavesurfer.drawer.drawPeaks(peaks, waveform.width() * wavesurfer.params.pixelRatio);
                        } else {
                            wavesurfer.load(tracks[n]['file']);
                        }
                        posterImg.animate({opacity: 1}, ANIMATION);
                        rightBox.animate({opacity: 1}, ANIMATION);
                        if (playnow || autoplay) obj.play();
                    }
                });
            } else {
                $.ajax({
                    url: waveplayer_ajax.ajax_url,
                    type: 'post',
                    data: [ {name: 'action', value: 'waveplayer_read_mp3_tags'}, {name: 'mp3', value: tracks[n]['file']} ],
                    dataType: 'json',
                    success: function(result) {
                        var file = tracks[n]['file'];
                        n = 0;
                        tracks[n] = result.data.tags;
                        
                        tracks[n]['file'] = file;
                        audio[0].src = tracks[n]['file'];
                        audio[0].load();
                        infoText.html( displayInfo( tracks[n] ) );
                        wavesurfer.load(tracks[n]['file']);
                        posterImg.animate({opacity: 1}, ANIMATION);
                        rightBox.animate({opacity: 1}, ANIMATION);
                        if (playnow || autoplay) obj.play();
                    }
                });
            }
        }
        
        /**
        *
        * Toggle the playback status
        *
        */
        this.toggle = function() {
            switch (status) {
                case 'play':
                    obj.pause();
                    break;
                case 'stop':
                case 'pause':
                    obj.play();
                    break;
            }
        };
                
        /**
        *
        * Loads the playlist for this instance of the player
        *
        */
        var loadPlaylist = function() {
            var type = 'playlist',
                ids = getData('ids'),
                mp3 = getData('mp3');
            
            if (ids) {
                $.ajax({
                    url: waveplayer_ajax.ajax_url,
                    type: 'post',
                    data: [ {name: 'action', value: 'waveplayer_load_playlist'}, {name: 'type', value: type}, {name: 'param', value: ids} ],
                    dataType: 'json',
                    success: function(result) {
                        if (result.data.tracks.length > 0) {
                            tracks = result.data.tracks;
                            playBtn.removeClass('wvpl-disabled');
                            checkSkipState();
                            status = 'stop';
                            wavesurfer.drawer.playing = false;
                            n = 0;
                            obj.preloadTrack();
                        } else {
                            playBtn.addClass('wvpl-disabled');
                        }
                    },
                    error: function() {
                        
                    }
                });
            } else {
                tracks = [];
                tracks[0] = {file: getData('mp3')};
                status = 'stop';
                wavesurfer.drawer.playing = false;
                n = 0;
                playBtn.removeClass('wvpl-disabled');
                obj.preloadTrack();
            }
        }
        
        /**
        *
        * Sets the visibility state of each skip button based on the current track in the playlist and the 'repeatAll' option
        *
        * @param playnow, if true, after preloading the track, automatically starts the playback
        *
        */
        var checkSkipState = function() {
            switch (n) {
                case 0:
                    if (!repeatAll) prevBtn.addClass('wvpl-disabled');
                    if (tracks.length > 1) nextBtn.removeClass('wvpl-disabled');
                    break;
                case 1:
                    if (tracks.length > 1) prevBtn.removeClass('wvpl-disabled');
                    if (tracks.length == 2 && !repeatAll) nextBtn.addClass('wvpl-disabled');
                    break;
                case tracks.length - 2:
                    if (tracks.length > 1) nextBtn.removeClass('wvpl-disabled');
                    break;
                case tracks.length - 1:
                    if (!repeatAll) nextBtn.addClass('wvpl-disabled');
                    if (tracks.length > 1) prevBtn.removeClass('wvpl-disabled');
                    break;
                default:
                    prevBtn.removeClass('wvpl-disabled');
                    nextBtn.removeClass('wvpl-disabled');
            }
        }
        
        /**
        *
        * Displays the information of the track provided
        *
        * @param track, an object containing the track whose info are going to be displayed
        *
        */
        var displayInfo = function(track) {
        
            var message = options.template,
                title = track['title'] == null ? 'no title' : track['title'],
                artist = track['artist'] == null ? '' : track['artist'],
                album = track['album'] == null ? '' : track['album'],
                comment = track['comment'] == null ? '' : track['comment'],
                length = track['length'] == null ? '' : track['length'],
                poster = track['poster'] == null ? '' : 'url('+track['poster']+')',
                tags = '';
                
                

/** ADD THIS WHEN THE TAG PART OF THE PLUGIN IS COMPLETED
*
*                $.each( tracks[n]['tags'], function( index, value ){
*                    tags += '<a href="/tag/' + value + '">#' + value + '</a>&nbsp;';
*                });
*
*/

            posterImg.css('background-image', poster);
            if (typeof(message) == 'undefined') {
                message = title;
            } else {
                var regex;
                for(var key in track){
                    regex = new RegExp('%'+key+'%', 'g');
                    message = message.replace(regex, track[key]);
                }
            }
            
            return message;
        }
        
                
        /**
        *
        * Bind the 'ended' event of the audio element to skip to the next track
        * and fires an 'ended' event
        *
        */
        audio.on('ended', function(event) {
            obj.skip();
            var track = tracks[n];
            elem.trigger('ended', {track: tracks[n]});
        });
        
        audio.on('canplay', function(event) {
            $('.wvpl-volume-overlay').html(Math.round($(this)[0].volume * 100));
        });
        
        audio.on('play', function(event) {
            timerHook = setInterval(timeUpdate, INTERVAL);
            var track = tracks[n];
            elem.trigger('play', {track: tracks[n]});
        });
        
        audio.on('pause', function(event) {
            clearInterval(timerHook);
            status = 'pause';
            wavesurfer.drawer.playing = false;
            var track = tracks[n];
            elem.trigger('pause', {track: tracks[n]});
        });
        
        var scrolling = false;
        infoText.on('mouseenter', function() {
            var el = $(this).find('.wvpl-infoblock'),
                scrollable = el.width() >= infoBar.width();
            if (scrollable && !scrolling) {
                scrolling = true;
                var elClone = el.clone().css({marginLeft:'1em'});
                $(this).append(elClone);
                var dV = elClone.outerWidth(true),
                    dT = 50 * dV;
                $(this).animate({left: -dV}, dT, 'linear', function() {
                        $(this).css({left: 0});
                        elClone.remove();
                        elem.trigger('titleScrollEnd', {track: tracks[n]});
                        scrolling = false;
                });
                elem.trigger('titleScrollStart', {track: tracks[n]});
                elClone.animate({left: -dV}, dT, 'linear');
            }
        });
        
        
        var alternate = (function () {
            var i = -1;
            return function () {
                return i++ == 1 ? i = 0 : i;  // change the 1 to 2 for a 3-state alternation
            }
        })();
        
        infoBtn.click(function() {
            switch (j = alternate()) {
                // No info
                case 0:
                    infoBar.addClass('wvpl-inactive').css({flexGrow:0});
                    infoBar.find('.wvpl-playing-extrainfo').addClass('wvpl-inactive');
                    break;
                // Title only
                case 1:
                    infoBar.removeClass('wvpl-inactive');
                    break;
                // Full Panel
                case 2:
                    infoBar.css({flexGrow:1}).find('.wvpl-playing-extrainfo').removeClass('wvpl-inactive');
                    break;
            }
            console.log(j);
        });
        
        playBtn.click(function() {
            obj.toggle();
        });
        
        prevBtn.click(function() {
            if (!$(this).hasClass('wvpl-disabled')) obj.skip(false);
        });
        
        nextBtn.click(function() {
            if (!$(this).hasClass('wvpl-disabled')) obj.skip();
        });
        
        $(window).resize(function() {
            setTimeout(waveRedraw, 500);
        });
        
        
        /*
        *
        * Makes the volume button draggable (yet not moving it)
        * so that the user can alter the volume clicking on the volume button
        * and dragging horizontally
        *
        */
        
        var timerOverlay,
            startOffset,
            startVol;
        volumeBtn.on('mousedown', function(e) {
            startOffset = e.clientX;
            startVol = audio[0].volume;
            
            // The overlay displaying the level appears after the user holds the mouse for more than 150ms
            timerOverlay = setTimeout(function() {
                
                // Disables the default behavior of the click and drag gesture
                e.preventDefault();
                elem.find('.wvpl-volume-overlay').addClass('dragging');
                elem.find('.wvpl-controls').addClass('wvpl-inactive');
                $(window)
                    .on('mousemove', function(e) {
                        e.preventDefault();
                        volumeBtn.addClass('dragging');
                        var newVol = Math.round( 100 * ( startVol + ( e.clientX - startOffset ) / ( interface.width() * wavesurfer.params.pixelRatio / 4 ) ) ) / 100;
                        if ( newVol >= 0 && newVol <= 1 ) {
                            audio[0].volume = newVol;
                            elem.find('.wvpl-volume-overlay').html(Math.round(newVol*100));
                            audio[0].muted = (newVol == 0);
                            newVol == 0 ?
                                volumeBtn.find('.wvpl-icon').removeClass('fa-volume-up').addClass('fa-volume-off') :
                                volumeBtn.find('.wvpl-icon').removeClass('fa-volume-off').addClass('fa-volume-up');
                        }
                    }).on('mouseup', function(e) {
                        $(window).off('mousemove');
                        $('.dragging').removeClass('dragging');
                        elem.find('.wvpl-controls').removeClass('wvpl-inactive');
                    });
            }, 150);
        }).on('mouseup', function() {
            clearTimeout(timerOverlay);
            if ( !volumeBtn.hasClass('dragging') ) {
                volumeBtn.find('.wvpl-icon').toggleClass('fa-volume-off').toggleClass('fa-volume-up');
                audio[0].muted = volumeBtn.find('.wvpl-icon').hasClass('fa-volume-off');
                elem.find('.wvpl-volume-overlay').html(Math.round(audio[0].volume * 100))
            }
            $('.dragging').removeClass('dragging');
        });
                
    };
    
    $.fn.waveplayer = function(params) {
        return this.each(function() {
           var element = $(this);
          
           // Return early if this element already has a plugin instance
           if (element.data('waveplayer')) return;

           var waveplayer = new WavePlayer(this);

           // Store plugin object in this element's data
           element.data('waveplayer', waveplayer);
           
           waveplayer.init(params);
        });
    };
    
    $('.waveplayer').waveplayer();
    
})(jQuery);