'use strict';

WaveSurfer.Drawer.Canvas = Object.create(WaveSurfer.Drawer);

WaveSurfer.util.extend(WaveSurfer.Drawer.Canvas, {
    createElements: function () {
        this.progress = 0;
        this.position = -1;
        this.playing = false;

        this.canvas = this.wrapper.appendChild(
            this.style(document.createElement('canvas'), {
                height: '100%',
                width: '100%',
            })
        );
        this.canvasCc = this.canvas.getContext('2d');
        this.canvas.addEventListener('mousemove', this.mousemove.bind(this), false);
        this.canvas.addEventListener('mouseleave', this.mouseleave.bind(this), false);

        var wave = document.createElement('canvas');
        this.waveCc = wave.getContext('2d');
        
        var progress = document.createElement('canvas');
        this.progressCc = progress.getContext('2d');
        
        var position = document.createElement('canvas');
        this.positionCc = position.getContext('2d');
        
    },
    
    mousemove: function(el) {
        if (this.playing) this.updatePosition(el.offsetX / ( this.width / this.params.pixelRatio ));
    },

    mouseleave: function(el) {
        if (this.playing) this.updatePosition(-1);
    },

    updateSize: function () {
        this.height = this.params.height * this.params.pixelRatio;
        this.canvasCc.canvas.width = this.waveCc.canvas.width = this.progressCc.canvas.width = this.positionCc.canvas.width = this.width;
        this.canvasCc.canvas.height = this.waveCc.canvas.height = this.progressCc.canvas.height = this.positionCc.canvas.height = this.height;
    },
    
    clearWave: function () {
        this.progress = 0;
        this.position = -1;
        this.canvasCc.clearRect(0, 0, this.width, this.height);
        this.waveCc.clearRect(0, 0, this.width, this.height);
        this.progressCc.clearRect(0, 0, this.width, this.height);
        this.positionCc.clearRect(0, 0, this.width, this.height);
    },

    drawBars: function (peaks) {
        this.peaks = peaks;
        
        if (typeof(peaks) == 'undefined') return false;
        
        var width = this.width,
            height = this.params.height * this.params.pixelRatio,
            length = ~~(peaks.length / 2),
            bar = this.params.waveMode * this.params.pixelRatio,
            gap = this.params.gapWidth * this.params.pixelRatio, //Math.max(this.params.pixelRatio, ~~(bar / 2)),
            cl = this.params.pixelRatio,
            step = bar + gap,
            a = this.params.asymmetry,
            c = this.params.compression,
            scale = width / length,
            halfH = height / (1 + 1/a);

        var absmax = 1;
        if (this.params.normalize) {
            var min, max;
            max = Math.max.apply(Math, peaks);
            min = Math.min.apply(Math, peaks);
            absmax = max;
            if (-min > absmax) {
                absmax = -min;
            }
        }
        
        var waves = [];
        waves.push({cc: this.waveCc, color1: this.params.waveColor, color2: this.params.waveColor2, alpha: 1 });
        waves.push({cc: this.progressCc, color1: this.params.progressColor, color2: this.params.progressColor2, alpha: 1 });
        waves.push({cc: this.positionCc, color1: this.params.progressColor, color2: this.params.progressColor2, alpha: this.params.hoverOpacity/100});

        waves.forEach(function (el) {
            var cc = el.cc,
                halfH = el.cc.canvas.height / ( 1 + 1 / this.params.asymmetry);
            if (!cc) { return; }

            var grd = this.waveCc.createLinearGradient(0, 0, 0, halfH);
            grd.addColorStop(0, el.color1);
            grd.addColorStop(1, typeof(el.color2) != 'undefined' ? el.color2 : el.color1);
            cc.fillStyle = grd;
            cc.globalAlpha = el.alpha;
            
            for (var i = 0; i < width; i += step) {
                var peak = peaks[Math.floor(2 * i / scale)];
                peak = Math.pow(Math.abs(peak), 1/c);
                var h = halfH * peak / absmax;
                h = Math.max(this.params.pixelRatio, Math.abs(h));
                cc.fillRect(i, halfH - h - cl/2, bar, h);
                cc.save();
                cc.fillStyle = el.color1;
                cc.fillRect(i, halfH + cl/2, bar, h * 1/a);
                cc.restore();
            }
        }, this);
        this.canvasCc.drawImage(this.waveCc.canvas, 0, 0);
        this.fireEvent('waveComplete');
    },

    drawWave: function (peaks, channelIndex) {
        if (typeof(peaks) == 'undefined') return false;
        
        this.peaks = peaks;
        
        var width = this.width,
            height = this.params.height * this.params.pixelRatio,
            length = ~~(peaks.length / 2),
            cl = this.params.pixelRatio,
            a = this.params.asymmetry,
            c = this.params.compression,
            scale = width / length,
            halfH = height / (1 + 1/a);

        var absmax = 1;
        if (this.params.normalize) {
            var min, max;
            max = Math.max.apply(Math, peaks);
            min = Math.min.apply(Math, peaks);
            absmax = max;
            if (-min > absmax) {
                absmax = -min;
            }
        }
        
        var waves = [];
        waves.push({cc: this.waveCc, color1: this.params.waveColor, color2: this.params.waveColor2, alpha: 1 });
        waves.push({cc: this.progressCc, color1: this.params.progressColor, color2: this.params.progressColor2, alpha: 1 });
        waves.push({cc: this.positionCc, color1: this.params.progressColor, color2: this.params.progressColor2, alpha: this.params.hoverOpacity/100}); 

        waves.forEach(function (el) {
            var cc = el.cc,
                halfH = el.cc.canvas.height / ( 1 + 1 / this.params.asymmetry);
            if (!cc) { return; }

            var grd = el.cc.createLinearGradient(0, 0, 0, halfH);
            grd.addColorStop(0, el.color1);
            grd.addColorStop(1, typeof(el.color2) != 'undefined' ? el.color2 : el.color1);
            cc.fillStyle = grd;
            cc.globalAlpha = el.alpha;
            
            if (this.params.reflection) {
                cc.beginPath();
                cc.moveTo(0, halfH);            
                for (var i = 0; i < length; i++) {
                    var peak = peaks[2 * i];
                    peak = Math.pow(Math.abs(peak), 1/c);
                    var h = peak / absmax * halfH;
                    cc.lineTo(i * scale, halfH - h);
                }
                cc.lineTo(el.cc.canvas.width, halfH);
                cc.closePath();
                cc.fill();
                cc.fillStyle = el.color1;
                cc.beginPath();
                cc.moveTo(el.cc.canvas.width, halfH);            
                for (var i = length - 1; i >= 0; i--) {
                    var peak = peaks[2 * i];
                    peak = Math.pow(Math.abs(peak), 1/c);
                    var h = peak / absmax * halfH;
                    cc.lineTo(i * scale, halfH + h * 1/a);
                }
                cc.lineTo(0, halfH);
                cc.closePath();
                cc.fill();
            } else {
                cc.beginPath();
                cc.moveTo(0, halfH);            
                for (var i = 0; i < length; i++) {
                    var peak = peaks[2 * i];
                    peak = Math.pow(Math.abs(peak), 1/c);
                    var h = peak / absmax * halfH;
                    cc.lineTo(i * scale, halfH - h);
                }
                cc.closePath();
                cc.fill();
                cc.fillStyle = el.color1;
                cc.beginPath();
                cc.moveTo(el.cc.canvas.width, halfH);            
                for (var i = length - 1; i >= 0; i--) {
                    var peak = peaks[2 * i + 1];
                    peak = Math.pow(Math.abs(peak), 1/c);
                    var h = peak / absmax * halfH;
                    cc.lineTo(i * scale, halfH + h * 1/a);
                }
                cc.closePath();
                cc.fill();
            }


            // Always draw a median line
//            cc.fillStyle = 'rgba(255,255,255,1)';
//            cc.fillRect(0, halfH - 1, this.width, 2);
        }, this);
        this.canvasCc.drawImage(this.waveCc.canvas, 0, 0);
        this.fireEvent('waveComplete');
    },

    updateProgress: function (progress) {
        this.progress = progress;
        var prog = Math.round(this.width * this.progress);
        var pos = Math.round(this.width * this.position);
        this.redrawContexts(prog, pos);
    },
    
    updatePosition: function (position) {
        var prog = Math.round(this.width * this.progress);
        this.position = position;
        var pos = Math.round(this.width * position);
        this.redrawContexts(prog, pos);
    },
    
    redrawContexts: function(prog, pos) {
        var self = this;
            
        var redraw = function(s1, e1, s2, e2) {
            cc.save();
            cc.beginPath();
            cc.rect(s1, 0, e1, self.height);
            cc.clip();
            cc.drawImage(self.progressCc.canvas, 0, 0);
            cc.restore();
            cc.save();
            cc.beginPath();
            cc.rect(s2, 0, e2, self.height);
            cc.clip();
            cc.drawImage(self.positionCc.canvas, 0, 0);
            cc.restore();
            
        }
        
        var cc = this.canvasCc;
        
        cc.clearRect(0, 0, this.width, this.height);
        cc.drawImage(this.waveCc.canvas, 0, 0);
        if ( pos < 0 ) {
            redraw(0, prog, 0, 0);
        } else {
            if (prog > pos) {
                redraw(0, pos, pos, prog - pos);
            } else {
                redraw(0, prog, prog, pos - prog);
            }
        }
        this.drawCursor(prog);
    },
    
    drawCursor: function(prog) {
        if (this.playing) {
            var halfH = ( this.params.height * this.params.pixelRatio ) / (1 + 1 / this.params.asymmetry);
            var t = this.params.cursorWidth * this.params.pixelRatio;
            var cc = this.canvasCc;
            var grd = this.waveCc.createLinearGradient(0, 0, 0, this.height);
            grd.addColorStop(0, this.params.cursorColor);
            grd.addColorStop(1, typeof(this.params.cursorColor2) != 'undefined' ? this.params.cursorColor2 : this.params.cursorColor);
            cc.fillStyle = grd;
            cc.fillRect(prog - t/2, 0, t, this.height);
        }
    },
    
    colorLuminance: function(hex, lum) {

        // validate hex string
        hex = String(hex).replace(/[^0-9a-f]/gi, '');
        if (hex.length < 6) {
            hex = hex[0]+hex[0]+hex[1]+hex[1]+hex[2]+hex[2];
        }
        lum = lum || 0;

        // convert to decimal and change luminosity
        var rgb = "#", c, i;
        for (i = 0; i < 3; i++) {
            c = parseInt(hex.substr(i*2,2), 16);
            c = Math.round(Math.min(Math.max(0, c + (c * lum)), 255)).toString(16);
            rgb += ("00"+c).substr(c.length);
        }

        return rgb;
    }
    
});
