'use strict';

WaveSurfer.Drawer = {
    init: function (container, params) {
        this.container = container;
        this.params = params;

        this.width = 0;
        this.height = params.height * params.pixelRatio;

        this.lastPos = 0;
        this.createWrapper();
        this.createElements();
    },

    createWrapper: function () {
        this.wrapper = this.container;
        this.setupWrapperEvents();
    },

    handleEvent: function (e) {
        e.preventDefault();
        var bbox = this.wrapper.getBoundingClientRect();
        return ((e.clientX - bbox.left + this.wrapper.scrollLeft) / this.wrapper.scrollWidth) || 0;
    },

    setupWrapperEvents: function () {
        var my = this;

        this.wrapper.addEventListener('click', function a(e) {
            var scrollbarHeight = my.wrapper.height - my.wrapper.clientHeight;
            if (scrollbarHeight != 0) {
                // scrollbar is visible.  Check if click was on it
                var bbox = my.wrapper.getBoundingClientRect();
                if (e.clientY >= bbox.bottom - scrollbarHeight) {
                    // ignore mousedown as it was on the scrollbar
                    return;
                }
            }
            
            if (this.playing) my.updateProgress(my.params.pixelRatio * e.offsetX / my.width);
            if (my.params.interact) {
                my.fireEvent('click', e, my.handleEvent(e));
            }
        });

        this.wrapper.addEventListener('scroll', function a(e) {
            my.fireEvent('scroll', e);
        });
    },

    drawPeaks: function (peaks, length) {
        this.resetScroll();
        this.setWidth(length);

        this.params.waveMode ?
            this.drawBars(peaks) :
            this.drawWave(peaks);
    },

    style: function (el, styles) {
        Object.keys(styles).forEach(function (prop) {
            if (el.style[prop] !== styles[prop]) {
                el.style[prop] = styles[prop];
            }
        });
        return el;
    },

    resetScroll: function () {
        if (this.wrapper !== null) {
            this.wrapper.scrollLeft = 0;
        }
    },

//    recenter: function (percent) {
//        var position = this.wrapper.scrollWidth * percent;
//        this.recenterOnPosition(position, true);
//    },

//    recenterOnPosition: function (position, immediate) {
//        var scrollLeft = this.wrapper.scrollLeft;
//        var half = ~~(this.wrapper.clientWidth / 2);
//        var target = position - half;
//        var offset = target - scrollLeft;
//        var maxScroll = this.wrapper.scrollWidth - this.wrapper.clientWidth;

//        if (maxScroll == 0) {
//            // no need to continue if scrollbar is not there
//            return;
//        }

//        // if the cursor is currently visible...
//        if (!immediate && -half <= offset && offset < half) {
//            // we'll limit the "re-center" rate.
//            var rate = 5;
//            offset = Math.max(-rate, Math.min(rate, offset));
//            target = scrollLeft + offset;
//        }

//        // limit target to valid range (0 to maxScroll)
//        target = Math.max(0, Math.min(maxScroll, target));
//        // no use attempting to scroll if we're not moving
//        if (target != scrollLeft) {
//            this.wrapper.scrollLeft = target;
//        }

//    },

    getWidth: function () {
        return Math.round(this.container.clientWidth * this.params.pixelRatio);
    },

    setWidth: function (width) {
        if (width == this.width) { return; }

        this.width = width;

        if (this.params.fillParent || this.params.scrollParent) {
            this.style(this.wrapper, {
                width: ''
            });
        } else {
            this.style(this.wrapper, {
                width: ~~(this.width / this.params.pixelRatio) + 'px'
            });
        }

        this.updateSize();
    },

    setHeight: function (height) {
        if (height == this.height) { return; }
        this.height = height;
        this.style(this.wrapper, {
            height: ~~(this.height / this.params.pixelRatio) + 'px'
        });
        this.updateSize();
    },

    destroy: function () {
        this.unAll();
        if (this.wrapper) {
            this.container.removeChild(this.canvas);
            this.wrapper = null;
        }
    },

    /* Renderer-specific methods */
    createElements: function () {},

    updateSize: function () {},

    drawWave: function (peaks, max) {},

    clearWave: function () {},

    updateProgress: function (position) {}
};

WaveSurfer.util.extend(WaveSurfer.Drawer, WaveSurfer.Observer);
