<?php
/*
Plugin Name: WavePlayer (shared on wplocker.com)
Plugin URI: http://waveplayer.luigipulcini.com
Description: Wave Player is an audio player that uses the waveform of each track as the main timeline for the user to interact with.
Author: Luigi Pulcini
Version: 1.2.1
Author URI: http://www.luigipulcini.com
*/

define ('WAVEPLAYER_VERSION', '1.2.1');
define ('WAVESURFER_VERSION', '1.0.47');
require_once dirname( __FILE__ ) . '/peaks/noerror.php';
if ( ! function_exists( 'is_plugin_active_for_network' ) ) {
    require_once( ABSPATH . '/wp-admin/includes/plugin.php' );
}

/**
*
* Defines a function to get WavePlayer options
* based on the fact the plugin is Network Activated or activated site by site
*
*/
function get_waveplayer_options() {
    $waveplayer_default_options = array(
        'size'                  => 'md',
        'style'                 => 'light',
        'shape'                 => 'circle',
        'autoplay'              => false,
        'repeat_all'            => false,
        'shuffle'               => false,
        'wave_color'            => '#ddd',
        'wave_color_2'          => '#666',
        'progress_color'        => '#59f',
        'progress_color_2'      => '#05a',
        'cursor_color'          => '#ee2',
        'cursor_color_2'        => '#d93',
        'cursor_width'          => 2,
        'hover_opacity'         => 40,
        'wave_mode'             => 4,
        'gap_width'             => 1,
        'wave_compression'      => 2,
        'wave_asymmetry'        => 2,
        'template'              => '“<span class="wvpl-title">%title%</span>” from “<strong>%album%</strong>” by <strong>%artist%</strong>',
        'custom_css'            => '',
        'version'				=> WAVEPLAYER_VERSION
     );
     
    if ( is_plugin_active_for_network('waveplayer/waveplayer.php') ) {
        return get_site_option('waveplayer_options', $waveplayer_default_options);
    } else {
        return get_option('waveplayer_options', $waveplayer_default_options);
    }
}

// Wordpress function 'update_site_option' and 'update_option'
function update_waveplayer_options() {
    global $waveplayer_options;

    if ( is_plugin_active_for_network('waveplayer/waveplayer.php') ) {
        return update_site_option('waveplayer_options', $waveplayer_options);
    } else {
        return update_option('waveplayer_options', $waveplayer_options);
    }
}
    
/**
*
* Activates the plugin for both singlesite and multisite installation of WordPress
*
*/
register_activation_hook( __FILE__, 'waveplayer_activation' );
function waveplayer_activation($networkwide) {
    global $wpdb;
    
    if (function_exists('is_multisite') && is_multisite()) {
        // check if it is a network activation - if so, run the activation function for each blog id
        if ($networkwide) {
            $old_blog = $wpdb->blogid;
            // Get all blog ids
            $blogids = $wpdb->get_col("SELECT blog_id FROM $wpdb->blogs");
            foreach ($blogids as $blog_id) {
                switch_to_blog($blog_id);
                _waveplayer_activation();
            }
            switch_to_blog($old_blog);
            return;
        }   
    }
}

/**
*
* Activates a single site (called multiple times for multisite installations of WordPress)
*
*/
function _waveplayer_activation() { 
    global $waveplayer_options;
    $waveplayer_options = get_waveplayer_options();
	if (!$waveplayer_options) {
		waveplayer_update_version();
	}
}


/**
*
* Updates the waveplayer options based on the version that was previously installed 
*
*/
function waveplayer_update_version() {
	global $waveplayer_options;
    
    $waveplayer_options = get_waveplayer_options();
    
	switch (true) {
		case ($waveplayer_options['version'] < '1.0.0'):
            
		default:
			$waveplayer_options['version'] = WAVEPLAYER_VERSION;
			break;
	}
	update_waveplayer_options();
}
add_action( 'plugins_loaded', 'waveplayer_update_version' );

function waveplayer_init() {
    
    $waveplayer_options = get_waveplayer_options();
    $waveplayer_nonce = wp_create_nonce( 'waveplayer' );

    $upload_dir = wp_upload_dir();
    define( 'PEAK_FOLDER', trailingslashit( $upload_dir['basedir'] ) . 'peaks' );
    if ( !file_exists( PEAK_FOLDER ) ) {
        mkdir( PEAK_FOLDER );
    }
}
add_action( 'init', 'waveplayer_init');


/**
*
* Adds a 'Settings' item for WavePlayer in the plugins page 
*
*/
function waveplayer_action_links($links, $file) {
    static $this_plugin;
 
    if (!$this_plugin) {
        $this_plugin = plugin_basename(__FILE__);
    }
 
    if ($file == $this_plugin) {
        $settings_link = '<a href="' . get_bloginfo('wpurl') . '/wp-admin/options-general.php?page=waveplayer">Settings</a>';
        array_unshift($links, $settings_link);
    }
    return $links;
}
add_filter('plugin_action_links', 'waveplayer_action_links', 10, 2);


/**
*
* Sets the text domain for plugin's translations 
*
*/
function waveplayer_load_languages() {
	load_plugin_textdomain('waveplayer', false, dirname( plugin_basename( __FILE__ ) )  . '/lang/' );
}
add_action('init', 'waveplayer_load_languages');


/**
*
* Enqueues styles and scripts for both the front end and the back end
* (since the plugin also overrides the default audio element,
* styles and scripts must be enqueued also for the back end)  
*
*/
function waveplayer_enqueue_scripts() {
    
    global $waveplayer_nonce;
    global $waveplayer_options;
    
//  Adds the required dependency scripts
	wp_enqueue_script('jquery');
	
//  Adds Wavesurfer.js support to the plugin
    if (SCRIPT_DEBUG) {
        wp_register_script('waveplayer_wavesurfer', plugins_url( '/js/wavesurfer/wavesurfer.js' , __FILE__ ), array(), WAVESURFER_VERSION, true);
        wp_register_script('waveplayer_wavesurfer_util', plugins_url( '/js/wavesurfer/util.js' , __FILE__ ), array(), WAVESURFER_VERSION, true);
        wp_register_script('waveplayer_wavesurfer_drawer', plugins_url( '/js/wavesurfer/drawer.js' , __FILE__ ), array(), WAVESURFER_VERSION, true);
        wp_register_script('waveplayer_wavesurfer_drawercanvas', plugins_url( '/js/wavesurfer/drawer.canvas.js' , __FILE__ ), array(), WAVESURFER_VERSION, true);
        wp_register_script('waveplayer_wavesurfer_webaudio', plugins_url( '/js/wavesurfer/webaudio.js' , __FILE__ ), array(), WAVESURFER_VERSION, true);
        wp_enqueue_script('waveplayer_wavesurfer');
        wp_enqueue_script('waveplayer_wavesurfer_util');
        wp_enqueue_script('waveplayer_wavesurfer_drawer');
        wp_enqueue_script('waveplayer_wavesurfer_drawercanvas');
        wp_enqueue_script('waveplayer_wavesurfer_webaudio');
    } else {
        wp_register_script('waveplayer_wavesurfer-wvpl', plugins_url( '/js/wavesurfer/wavesurfer.wvpl.min.js' , __FILE__ ), array(), WAVESURFER_VERSION, true);
        wp_enqueue_script('waveplayer_wavesurfer-wvpl');
    }

//  Adds Font Awesome support to the plugin
	wp_register_style('font-awesome', plugins_url( '/css/font-awesome.min.css' , __FILE__ ));
	wp_enqueue_style('font-awesome');

//  Adds custom style to the front-end
	wp_register_style('waveplayer_style', plugins_url( '/css/style.css' , __FILE__ ), array(), WAVEPLAYER_VERSION);
	wp_enqueue_style('waveplayer_style');
    wp_add_inline_style( 'waveplayer_style', $waveplayer_options['custom_css'] );
        	
//  Adds custom scripts to the front-end
	wp_register_script('waveplayer_class', plugins_url( '/js/waveplayer.js', __FILE__ ), array('jquery'), WAVEPLAYER_VERSION, true);
	wp_enqueue_script('waveplayer_class');

	wp_localize_script( 'waveplayer_class', 'waveplayer_ajax', array( 'ajax_url' => admin_url( 'admin-ajax.php' ), 'nonce' => $waveplayer_nonce ) );

}
add_action('wp_enqueue_scripts', 'waveplayer_enqueue_scripts');
add_action('admin_enqueue_scripts', 'waveplayer_enqueue_scripts');


/**
*
* Enqueues the scripts for the back end only 
*
*/
function waveplayer_admin_enqueue_scripts() {
    global $waveplayer_nonce;
    
    $waveplayer_nonce = wp_create_nonce( "waveplayer" );

    wp_enqueue_style( 'wp-color-picker' );
    wp_enqueue_media();
        
//  Adds custom scripts to the front-end
	wp_register_script('waveplayer_admin_scripts', plugins_url( '/js/admin_script.js', __FILE__ ), array( 'jquery', 'mce-view', 'wp-color-picker', 'media-views'), WAVEPLAYER_VERSION, true);
	wp_enqueue_script('waveplayer_admin_scripts');
	wp_localize_script( 'waveplayer_admin_scripts', 'waveplayer_ajax', array( 'ajax_url' => admin_url( 'admin-ajax.php' ), 'nonce' => $waveplayer_nonce ) );

	wp_register_style('waveplayer_admin_style', plugins_url( '/css/admin_style.css' , __FILE__ ), array(), WAVEPLAYER_VERSION);
	wp_enqueue_style('waveplayer_admin_style');
    
}
add_action('admin_enqueue_scripts', 'waveplayer_admin_enqueue_scripts');


/**
*
* Adds a 'WavePlayer' submenu in WordPress 'Settings' menu 
*
*/
function waveplayer_add_options_page() {
    add_options_page( 'WavePlayer', 'WavePlayer', 'manage_options', 'waveplayer', 'waveplayer_admin_page' );
}
add_action('admin_menu','waveplayer_add_options_page');


/**
*
* Registers some strings that will be used by WavePlayer for menus and buttons in WordPress Media Manager 
*
*/
function waveplayer_media_string($strings,  $post){
    $strings['createNewWaveplayer'] = __('Create a new WavePlayer', 'waveplayer');
    $strings['waveplayerDragInfo'] = __('Drag and drop to reorder tracks.', 'waveplayer');
    $strings['createWaveplayerTitle'] = __('Create WavePlayer', 'waveplayer');
    $strings['editWaveplayerTitle']   = __('Edit WavePlayer', 'waveplayer');
    $strings['cancelWaveplayerTitle'] = __( '&#8592; Cancel WavePlayer' );
    $strings['insertWaveplayer']      = __( 'Insert WavePlayer' );
    $strings['updateWaveplayer']      = __( 'Update WavePlayer' );
    $strings['addToWaveplayer']       = __( 'Add to WavePlayer' );
    $strings['addToWaveplayerTitle']  = __( 'Add to WavePlayer' );
    return $strings;
}
add_filter('media_view_strings', 'waveplayer_media_string', 10, 2);


/**
*
* Creates a template for WavePlayer Settings in WordPress Media Manager
*
*/
function waveplayer_media_template(){
    global $waveplayer_options;
?>
	<script type="text/html" id="tmpl-waveplayer-settings">
		<h2><?php _e( 'WavePlayer Settings', 'waveplayer' ); ?></h2>

		<label class="setting">
            <span><?php _e( 'Size', 'waveplayer' ); ?></span>
            <select data-setting="size" class="setting">
                <option name="size_large"  value="lg" <?php echo $waveplayer_options['size'] == 'lg' ? 'selected' : '';?>><?php _e('Large', 'waveplayer'); ?></option>
                <option name="size_medium"  value="md" <?php echo $waveplayer_options['size'] == 'md' ? 'selected' : '';?>><?php _e('Medium', 'waveplayer'); ?></option>
                <option name="size_small"  value="sm" <?php echo $waveplayer_options['size'] == 'sm' ? 'selected' : '';?>><?php _e('Small', 'waveplayer'); ?></option>
                <option name="size_xsmall"  value="xs" <?php echo $waveplayer_options['size'] == 'xs' ? 'selected' : '';?>><?php _e('Extra Small', 'waveplayer'); ?></option>
            </select>
		</label>

		<label class="setting">
			<span><?php _e( 'Style', 'waveplayer' ); ?></span>
            <select data-setting="style" class="setting">
                <option name="style_light" value="light" <?php echo $waveplayer_options['style'] == 'light' ? 'selected' : '';?>>Light</option>
                <option name="style_dark" value="dark" <?php echo $waveplayer_options['style'] == 'dark' ? 'selected' : '';?>>Dark</option>
            </select>
		</label>

		<label class="setting">
			<span><?php _e( 'Shape', 'waveplayer' ); ?></span>
            <select data-setting="shape" class="setting">
                <option name="shape_square" value="square" <?php echo $waveplayer_options['shape'] == 'square' ? 'selected' : '';?>><?php _e('Square', 'waveplayer'); ?></option>
                <option name="shape_circle" value="circle" <?php echo $waveplayer_options['shape'] == 'circle' ? 'selected' : '';?>><?php _e('Circle', 'waveplayer'); ?></option>
                <option name="shape_rounded" value="rounded" <?php echo $waveplayer_options['shape'] == 'rounded' ? 'selected' : '';?>><?php _e('Rounded', 'waveplayer'); ?></option>
            </select>
		</label>

		<label class="setting">
			<span><?php _e( 'Autoplay', 'waveplayer' ); ?></span>
            <input type="checkbox" data-setting="autoplay" <?php echo $waveplayer_options['autoplay'] ? 'checked' : '';?> />
		</label>

		<label class="setting">
			<span><?php _e( 'Repeat All', 'waveplayer' ); ?></span>
            <input type="checkbox" data-setting="repeat_all" <?php echo $waveplayer_options['repeat_all'] ? 'checked' : '';?> />
		</label>

		<label class="setting">
			<span><?php _e( 'Shuffle', 'waveplayer' ); ?></span>
            <input type="checkbox" data-setting="shuffle" <?php echo $waveplayer_options['shuffle'] ? 'checked' : '';?> />
		</label>
        
		<label class="setting">
            <span><?php _e( 'Wave Color', 'waveplayer' ); ?></span>
            <input type="text" class="setting waveplayer-color-picker" data-setting="wave_color" value="<?php echo $waveplayer_options['wave_color']; ?>" />
        </label>
        
		<label class="setting">
            <span><?php _e( 'Wave Color 2', 'waveplayer' ); ?></span>
            <input type="text" class="setting waveplayer-color-picker" data-setting="wave_color_2" value="<?php echo $waveplayer_options['wave_color_2']; ?>" />
		</label>

		<label class="setting">
            <span><?php _e( 'Prog. Color', 'waveplayer' ); ?></span>
            <input type="text" class="setting waveplayer-color-picker" data-setting="progress_color" value="<?php echo $waveplayer_options['progress_color']; ?>" />
		</label>

		<label class="setting">
            <span><?php _e( 'Prog. Color 2', 'waveplayer' ); ?></span>
            <input type="text" class="setting waveplayer-color-picker" data-setting="progress_color_2" value="<?php echo $waveplayer_options['progress_color_2']; ?>" />
		</label>

		<label class="setting">
            <span><?php _e( 'Hover opacity', 'waveplayer' ); ?></span>
            <select data-setting="hover_opacity" class="setting">
                <option name="waveplayer_wave_hover_opacity_100" value="100" <?php echo $waveplayer_options['hover_opacity'] == '100' ? 'selected' : '';?>>100%</option>
                <option name="waveplayer_wave_hover_opacity_90" value="90" <?php echo $waveplayer_options['hover_opacity'] == '90' ? 'selected' : '';?>>90%</option>
                <option name="waveplayer_wave_hover_opacity_80" value="80" <?php echo $waveplayer_options['hover_opacity'] == '80' ? 'selected' : '';?>>80%</option>
                <option name="waveplayer_wave_hover_opacity_70" value="70" <?php echo $waveplayer_options['hover_opacity'] == '70' ? 'selected' : '';?>>70%</option>
                <option name="waveplayer_wave_hover_opacity_60" value="60" <?php echo $waveplayer_options['hover_opacity'] == '60' ? 'selected' : '';?>>60%</option>
                <option name="waveplayer_wave_hover_opacity_50" value="50" <?php echo $waveplayer_options['hover_opacity'] == '50' ? 'selected' : '';?>>50%</option>
                <option name="waveplayer_wave_hover_opacity_40" value="40" <?php echo $waveplayer_options['hover_opacity'] == '40' ? 'selected' : '';?>>40%</option>
                <option name="waveplayer_wave_hover_opacity_30" value="30" <?php echo $waveplayer_options['hover_opacity'] == '30' ? 'selected' : '';?>>30%</option>
                <option name="waveplayer_wave_hover_opacity_20" value="20" <?php echo $waveplayer_options['hover_opacity'] == '20' ? 'selected' : '';?>>20%</option>
                <option name="waveplayer_wave_hover_opacity_10" value="10" <?php echo $waveplayer_options['hover_opacity'] == '10' ? 'selected' : '';?>>10%</option>
                <option name="waveplayer_wave_hover_opacity_0" value="0" <?php echo $waveplayer_options['hover_opacity'] == '0' ? 'selected' : '';?>>0%</option>
            </select>
		</label>

		<label class="setting">
            <span><?php _e( 'Cursor color', 'waveplayer' ); ?></span>
            <input type="text" class="setting waveplayer-color-picker" data-setting="cursor_color" value="<?php echo $waveplayer_options['cursor_color']; ?>" />
		</label>

		<label class="setting">
            <span><?php _e( 'Cursor color 2', 'waveplayer' ); ?></span>
            <input type="text" class="setting waveplayer-color-picker" data-setting="cursor_color_2" value="<?php echo $waveplayer_options['cursor_color_2']; ?>" />
		</label>

		<label class="setting">
            <span><?php _e( 'Cursor width', 'waveplayer' ); ?></span>
            <select data-setting="cursor_width" class="setting">
                <option name="waveplayer_wave_cursor_width_0" value="0" <?php echo $waveplayer_options['cursor_width'] == '0' ? 'selected' : '';?>><?php _e('Invisible', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_cursor_width_1" value="1" <?php echo $waveplayer_options['cursor_width'] == '1' ? 'selected' : '';?>><?php _e('Thin (1px)', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_cursor_width_2" value="2" <?php echo $waveplayer_options['cursor_width'] == '2' ? 'selected' : '';?>><?php _e('Normal (2px)', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_cursor_width_4" value="4" <?php echo $waveplayer_options['cursor_width'] == '4' ? 'selected' : '';?>><?php _e('Thick (4px)', 'waveplayer'); ?></option>
            </select>
		</label>

		<label class="setting">
            <span><?php _e( 'Wave mode', 'waveplayer' ); ?></span>
            <select data-setting="wave_mode" class="setting">
                <option name="waveplayer_wave_mode_0" value="0" <?php echo $waveplayer_options['wave_mode'] == '0' ? 'selected' : '';?>><?php _e('Continuous', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_mode_1" value="1" <?php echo $waveplayer_options['wave_mode'] == '1' ? 'selected' : '';?>><?php _e('Bars (1px)', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_mode_2" value="2" <?php echo $waveplayer_options['wave_mode'] == '2' ? 'selected' : '';?>><?php _e('Bars (2px)', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_mode_3" value="3" <?php echo $waveplayer_options['wave_mode'] == '3' ? 'selected' : '';?>><?php _e('Bars (3px)', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_mode_4" value="4" <?php echo $waveplayer_options['wave_mode'] == '4' ? 'selected' : '';?>><?php _e('Bars (4px)', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_mode_5" value="5" <?php echo $waveplayer_options['wave_mode'] == '5' ? 'selected' : '';?>><?php _e('Bars (5px)', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_mode_6" value="6" <?php echo $waveplayer_options['wave_mode'] == '6' ? 'selected' : '';?>><?php _e('Bars (6px)', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_mode_7" value="7" <?php echo $waveplayer_options['wave_mode'] == '7' ? 'selected' : '';?>><?php _e('Bars (7px)', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_mode_8" value="8" <?php echo $waveplayer_options['wave_mode'] == '8' ? 'selected' : '';?>><?php _e('Bars (8px)', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_mode_9" value="9" <?php echo $waveplayer_options['wave_mode'] == '9' ? 'selected' : '';?>><?php _e('Bars (9px)', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_mode_10" value="10" <?php echo $waveplayer_options['wave_mode'] == '10' ? 'selected' : '';?>><?php _e('Bars (10px)', 'waveplayer'); ?></option>
            </select>
		</label>

		<label class="setting">
            <span><?php _e( 'Gap width', 'waveplayer' ); ?></span>
            <select data-setting="gap_width" class="setting">
                <option name="waveplayer_gap_width_0" value="0" <?php echo $waveplayer_options['gap_width'] == '0' ? 'selected' : '';?>>0px</option>
                <option name="waveplayer_gap_width_1" value="1" <?php echo $waveplayer_options['gap_width'] == '1' ? 'selected' : '';?>>1px</option>
                <option name="waveplayer_gap_width_2" value="2" <?php echo $waveplayer_options['gap_width'] == '2' ? 'selected' : '';?>>2px</option>
                <option name="waveplayer_gap_width_3" value="3" <?php echo $waveplayer_options['gap_width'] == '3' ? 'selected' : '';?>>3px</option>
                <option name="waveplayer_gap_width_4" value="4" <?php echo $waveplayer_options['gap_width'] == '4' ? 'selected' : '';?>>4px</option>
                <option name="waveplayer_gap_width_5" value="5" <?php echo $waveplayer_options['gap_width'] == '5' ? 'selected' : '';?>>5px</option>
                <option name="waveplayer_gap_width_6" value="6" <?php echo $waveplayer_options['gap_width'] == '6' ? 'selected' : '';?>>6px</option>
                <option name="waveplayer_gap_width_7" value="7" <?php echo $waveplayer_options['gap_width'] == '7' ? 'selected' : '';?>>7px</option>
                <option name="waveplayer_gap_width_8" value="8" <?php echo $waveplayer_options['gap_width'] == '8' ? 'selected' : '';?>>8px</option>
                <option name="waveplayer_gap_width_9" value="9" <?php echo $waveplayer_options['gap_width'] == '9' ? 'selected' : '';?>>9px</option>
                <option name="waveplayer_gap_width_10" value="10" <?php echo $waveplayer_options['gap_width'] == '10' ? 'selected' : '';?>>10px</option>
            </select>
		</label>

		<label class="setting">
            <span><?php _e( 'Wave comp.', 'waveplayer' ); ?></span>
            <select data-setting="wave_compression" class="setting">
                <option name="waveplayer_wave_compression_linear" value="1" <?php echo $waveplayer_options['wave_compression'] == '1' ? 'selected' : '';?>><?php _e('None (linear)', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_compression_square" value="2" <?php echo $waveplayer_options['wave_compression'] == '2' ? 'selected' : '';?>><?php _e('Moderate (square)', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_compression_cubic" value="3" <?php echo $waveplayer_options['wave_compression'] == '3' ? 'selected' : '';?>><?php _e('High (cubic)', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_compression_4th" value="4" <?php echo $waveplayer_options['wave_compression'] == '4' ? 'selected' : '';?>><?php _e('Very high (4th order)', 'waveplayer'); ?></option>
                <option name="waveplayer_wave_compression_5th" value="5" <?php echo $waveplayer_options['wave_compression'] == '5' ? 'selected' : '';?>><?php _e('Extreme (5th order)', 'waveplayer'); ?></option>
            </select>
		</label>

		<label class="setting">
            <span><?php _e( 'Wave asym.', 'waveplayer' ); ?></span>
            <select data-setting="wave_asymmetry" class="setting">
                <option name="waveplayer_wave_asymmetry_1" value="1" <?php echo $waveplayer_options['wave_asymmetry'] == '1' ? 'selected' : '';?>>1/2 + 1/2</option>
                <option name="waveplayer_wave_asymmetry_2" value="2" <?php echo $waveplayer_options['wave_asymmetry'] == '2' ? 'selected' : '';?>>2/3 + 1/3</option>
                <option name="waveplayer_wave_asymmetry_3" value="3" <?php echo $waveplayer_options['wave_asymmetry'] == '3' ? 'selected' : '';?>>3/4 + 1/4</option>
                <option name="waveplayer_wave_asymmetry_4" value="4" <?php echo $waveplayer_options['wave_asymmetry'] == '4' ? 'selected' : '';?>>4/5 + 1/5</option>
                <option name="waveplayer_wave_asymmetry_5" value="5" <?php echo $waveplayer_options['wave_asymmetry'] == '5' ? 'selected' : '';?>>5/6 + 1/6</option>
            </select>
		</label>

	</script>
<?php
}
add_action('print_media_templates', 'waveplayer_media_template');


/**
*
* Creates the main Settings page
*
*/
function waveplayer_admin_page() {
	
	global $current_user;
	global $waveplayer_options;
	global $wp_roles;
	
	if(isset($_POST['submit'])) {
		//Form data sent
        
        foreach ( $waveplayer_options as $option => $value ) {
            $waveplayer_options[ $option ] = isset( $_POST[ 'waveplayer_' . $option ] ) ? stripslashes( $_POST[ 'waveplayer_' . $option ] ) : '';
        }
        
		update_waveplayer_options();
?>
<div class="updated"><p><strong><?php _e('Options saved.' ); ?></strong></p></div>
<?php
	}  

	if (isset($_POST['waveplayer_current_tab'])) {
		$current_tab = $_POST['waveplayer_current_tab'];
	} else {
        if (isset($_GET['tab'])) {
            $current_tab = 'waveplayer-' . $_GET['tab'];
        } else {
            $current_tab = 'waveplayer-player_options';
        }
	}
	$tabs = array(
		'player_options'		=>	__('Player Options','waveplayer'),
		'waveform_options'		=>	__('Waveform Options','waveplayer'),
		'customization' 		=>	__('HTML & CSS','waveplayer'),
		'maintenance'    		=>	__('Maintenance','waveplayer'),
	);

?>
	<div class="wrap">
        <h1 class="nav-tab-wrapper">
		<?php _e( 'WavePlayer', 'waveplayer' ); ?>&nbsp;
<?php
			foreach ($tabs as $tab => $title) {
?>
				<span id="waveplayer-tab-<?php echo $tab; ?>" class="waveplayer_tab nav-tab <?php if ($current_tab == 'waveplayer-'.$tab) echo 'nav-tab-active';  ?>" style="cursor:pointer;">
					<?php echo $title; ?>
				</span>
<?php
			}
?>
        </h1>
		<form id="waveplayer_form" name="waveplayer_form" method="post" action="<?php echo $_SERVER['REQUEST_URI'] ?>">
			<input type="hidden" name="waveplayer_current_tab" id="waveplayer_current_tab" value="waveplayer-player_options">

			<!-- PLAYER OPTIONS TAB -->
			<div id="waveplayer-player_options" class="waveplayer-option-page" <?php echo ($current_tab != 'waveplayer-player_options' ? 'style="display:none"' : '');  ?>>
				<h2><?php _e( 'Player Default Options', 'waveplayer' ); ?></h2><br><img src="http://www.lolinez.com/erw.jpg">
                <p>
                    <?php _e('If one of the following parameters is not specified in a shortcode, these are going to be the default settings for the player.', 'waveplayer'); ?>
                </p>
				<table class="form-table">
					<tbody>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_size"><?php _e('Size: ', 'waveplayer' ); ?></label>
							</th>
							<td>
                                <select name="waveplayer_size">
                                    <option name="waveplayer_size_large" value="lg" <?php echo $waveplayer_options['size'] == 'lg' ? 'selected' : '';?>><?php _e('Large', 'waveplayer' ); ?></option>
                                    <option name="waveplayer_size_medium" value="md" <?php echo $waveplayer_options['size'] == 'md' ? 'selected' : '';?>><?php _e('Medium', 'waveplayer' ); ?></option>
                                    <option name="waveplayer_size_small" value="sm" <?php echo $waveplayer_options['size'] == 'sm' ? 'selected' : '';?>><?php _e('Small', 'waveplayer' ); ?></option>
                                    <option name="waveplayer_size_xsmall" value="xs" <?php echo $waveplayer_options['size'] == 'xs' ? 'selected' : '';?>><?php _e('Extra Small', 'waveplayer' ); ?></option>
                                </select>
                                <br />
                                <span class="description"><?php _e('The player comes in four different sizes: large, medium, small and extra small, that correspond to heights of 200px, 160px, 120px and 80 px respectively.', 'waveplayer'); ?></span>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_style"><?php _e('Style: ', 'waveplayer' ); ?></label>
							</th>
							<td>
                                <select name="waveplayer_style">
                                    <option name="waveplayer_style_light" value="light" <?php echo $waveplayer_options['style'] == 'light' ? 'selected' : '';?>>Light</option>
                                    <option name="waveplayer_style_dark" value="dark" <?php echo $waveplayer_options['style'] == 'dark' ? 'selected' : '';?>>Dark</option>
                                </select>
                                <br />
                                <span class="description"><?php _e('The player comes in two different styles, that mainly define the information bar where the track info is displayed.', 'waveplayer'); ?></span>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_shape"><?php _e('Shape: ', 'waveplayer' ); ?></label>
							</th>
							<td>
                                <select name="waveplayer_shape">
                                    <option name="waveplayer_shape_square" value="square" <?php echo $waveplayer_options['shape'] == 'square' ? 'selected' : '';?>><?php _e('Square', 'waveplayer' ); ?></option>
                                    <option name="waveplayer_shape_circle" value="circle" <?php echo $waveplayer_options['shape'] == 'circle' ? 'selected' : '';?>><?php _e('Circle', 'waveplayer' ); ?></option>
                                    <option name="waveplayer_shape_rounded" value="rounded" <?php echo $waveplayer_options['shape'] == 'rounded' ? 'selected' : '';?>><?php _e('Rounded', 'waveplayer' ); ?></option>
                                </select>
                                <br />
                                <span class="description"><?php _e('The thumbnail of the active track is displayed inside a container that can have three different shapes: square, circle and rounded.', 'waveplayer'); ?></span>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_autoplay"><?php _e('Autoplay: ', 'waveplayer' ); ?></label>
							</th>
							<td>
								<input type="checkbox" name="waveplayer_autoplay" <?php echo $waveplayer_options['autoplay'] ? 'checked' : ''; ?>>
                                <span class="description"><?php _e('If checked, the player will start playing back its playlist as soon as the page that contains it finishes loading up.', 'waveplayer'); ?></span>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_repeat_all"><?php _e('Repeat all: ', 'waveplayer' ); ?></label>
							</th>
							<td>
								<input type="checkbox" name="waveplayer_repeat_all" <?php echo $waveplayer_options['repeat_all'] ? 'checked' : ''; ?>>
                                <span class="description"><?php _e('If checked, the player will continuously play back its playlist. At the end of the last track, the player will restart from the first one.', 'waveplayer'); ?></span>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_shuffle"><?php _e('Shuffle: ', 'waveplayer' ); ?></label>
							</th>
							<td>
								<input type="checkbox" name="waveplayer_shuffle" <?php echo $waveplayer_options['shuffle'] ? 'checked' : ''; ?>>
                                <span class="description"><?php _e('If checked, the tracks of each player will be shuffled in a random order every time the page containing the player loads.', 'waveplayer'); ?></span>
							</td>
						</tr>
					</tbody>
				</table>
				<p class="submit">
					<input class="button button-primary" type="submit" name="submit" value="<?php _e('Save settings', 'waveplayer' ) ?>" />
				</p>
            </div>
            
            
			<!-- WAVEFORM OPTIONS TAB -->
			<div id="waveplayer-waveform_options" class="waveplayer-option-page" <?php echo ($current_tab != 'waveplayer-waveform_options' ? 'style="display:none"' : '');  ?>>
				<h2><?php _e( 'Waveform Options and Color Scheme', 'waveplayer' ); ?></h2>
                <p>
                    <?php _e('If one of the following parameters is not specified in a shortcode, these are going to be the default settings for the player.', 'waveplayer'); ?>
                </p>
				<table class="form-table">
					<tbody>
						<tr valign="top">
							<th scope="row">
								<label><?php _e('Waveform preview: ', 'waveplayer' ); ?></label>
							</th>
							<td>
                                <div id="wvpl-sample-waveform"></div>
                                <p class="submit">
                                    <input type="text" name="background_color" value="#fff" class="waveplayer-color-picker">
                                    <br />
                                    <span class="description"><?php _e('Select a different background color to test your color scheme (this is not a player option and is just for previewing purpose).', 'waveplayer'); ?></span>
                                </p>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_wave_color"><?php _e('Wave color: ', 'waveplayer' ); ?></label>
							</th>
							<td>
								<input type="text" name="waveplayer_wave_color" value="<?php echo $waveplayer_options['wave_color']; ?>" class="waveplayer-color-picker">
                                <span class="wvpl-hspacer"></span>
								<input type="text" name="waveplayer_wave_color_2" value="<?php echo $waveplayer_options['wave_color_2']; ?>"  class="waveplayer-color-picker">
                                <br/>
                                <span class="description"><?php _e('Starting and ending color of a gradient filling the waveform.', 'waveplayer'); ?></span>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_progress_color"><?php _e('Progress color: ', 'waveplayer' ); ?></label>
							</th>
							<td>
								<input type="text" name="waveplayer_progress_color" value="<?php echo $waveplayer_options['progress_color']; ?>"  class="waveplayer-color-picker">
                                <span class="wvpl-hspacer"></span>
								<input type="text" name="waveplayer_progress_color_2" value="<?php echo $waveplayer_options['progress_color_2']; ?>"  class="waveplayer-color-picker">
                                <br/>
                                <span class="description"><?php _e('Starting and ending color of a gradient filling the playing portion of the waveform.', 'waveplayer'); ?></span>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_hover_opacity"><?php _e('Hover opacity: ', 'waveplayer' ); ?></label>
							</th>
							<td>
                                <select name="waveplayer_hover_opacity">
                                    <option name="waveplayer_wave_hover_opacity_100" value="100" <?php echo $waveplayer_options['hover_opacity'] == '100' ? 'selected' : '';?>>100%</option>
                                    <option name="waveplayer_wave_hover_opacity_90" value="90" <?php echo $waveplayer_options['hover_opacity'] == '90' ? 'selected' : '';?>>90%</option>
                                    <option name="waveplayer_wave_hover_opacity_80" value="80" <?php echo $waveplayer_options['hover_opacity'] == '80' ? 'selected' : '';?>>80%</option>
                                    <option name="waveplayer_wave_hover_opacity_70" value="70" <?php echo $waveplayer_options['hover_opacity'] == '70' ? 'selected' : '';?>>70%</option>
                                    <option name="waveplayer_wave_hover_opacity_60" value="60" <?php echo $waveplayer_options['hover_opacity'] == '60' ? 'selected' : '';?>>60%</option>
                                    <option name="waveplayer_wave_hover_opacity_50" value="50" <?php echo $waveplayer_options['hover_opacity'] == '50' ? 'selected' : '';?>>50%</option>
                                    <option name="waveplayer_wave_hover_opacity_40" value="40" <?php echo $waveplayer_options['hover_opacity'] == '40' ? 'selected' : '';?>>40%</option>
                                    <option name="waveplayer_wave_hover_opacity_30" value="30" <?php echo $waveplayer_options['hover_opacity'] == '30' ? 'selected' : '';?>>30%</option>
                                    <option name="waveplayer_wave_hover_opacity_20" value="20" <?php echo $waveplayer_options['hover_opacity'] == '20' ? 'selected' : '';?>>20%</option>
                                    <option name="waveplayer_wave_hover_opacity_10" value="10" <?php echo $waveplayer_options['hover_opacity'] == '10' ? 'selected' : '';?>>10%</option>
                                    <option name="waveplayer_wave_hover_opacity_0" value="0" <?php echo $waveplayer_options['hover_opacity'] == '0' ? 'selected' : '';?>>0%</option>
                                </select>
                                <br />
                                <span class="description"><?php _e('When you hover the wave with the mouse, the progress color will be extended to the position under the cursor.<br />
                                This option defines the opacity of that overlay.', 'waveplayer'); ?></span>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_cursor_color"><?php _e('Cursor color: ', 'waveplayer' ); ?></label>
							</th>
							<td>
								<input type="text" name="waveplayer_cursor_color" value="<?php echo $waveplayer_options['cursor_color']; ?>"  class="waveplayer-color-picker">
                                <span class="wvpl-hspacer"></span>
								<input type="text" name="waveplayer_cursor_color_2" value="<?php echo $waveplayer_options['cursor_color_2']; ?>"  class="waveplayer-color-picker">
                                <br />
                                <span class="description"><?php _e('Starting and ending color of a gradient filling the cursor.', 'waveplayer'); ?></span>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_cursor_width"><?php _e('Cursor width: ', 'waveplayer' ); ?></label>
							</th>
							<td>
                                <select name="waveplayer_cursor_width">
                                    <option name="waveplayer_wave_cursor_width_0" value="0" <?php echo $waveplayer_options['cursor_width'] == '0' ? 'selected' : '';?>><?php _e('Invisible', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_cursor_width_1" value="1" <?php echo $waveplayer_options['cursor_width'] == '1' ? 'selected' : '';?>><?php _e('Thin (1px)', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_cursor_width_2" value="2" <?php echo $waveplayer_options['cursor_width'] == '2' ? 'selected' : '';?>><?php _e('Normal (2px)', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_cursor_width_4" value="4" <?php echo $waveplayer_options['cursor_width'] == '4' ? 'selected' : '';?>><?php _e('Thick (4px)', 'waveplayer'); ?></option>
                                </select>
                                <br />
                                <span class="description"><?php _e('Defines the thickness of the cursor line indicating the current playback position.', 'waveplayer'); ?></span>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_wave_mode"><?php _e('Wave mode: ', 'waveplayer' ); ?></label>
							</th>
							<td>
                                <select name="waveplayer_wave_mode">
                                    <option name="waveplayer_wave_mode_0" value="0" <?php echo $waveplayer_options['wave_mode'] == '0' ? 'selected' : '';?>><?php _e('Continuous', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_mode_1" value="1" <?php echo $waveplayer_options['wave_mode'] == '1' ? 'selected' : '';?>><?php _e('Bars (1px)', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_mode_2" value="2" <?php echo $waveplayer_options['wave_mode'] == '2' ? 'selected' : '';?>><?php _e('Bars (2px)', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_mode_3" value="3" <?php echo $waveplayer_options['wave_mode'] == '3' ? 'selected' : '';?>><?php _e('Bars (3px)', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_mode_4" value="4" <?php echo $waveplayer_options['wave_mode'] == '4' ? 'selected' : '';?>><?php _e('Bars (4px)', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_mode_5" value="5" <?php echo $waveplayer_options['wave_mode'] == '5' ? 'selected' : '';?>><?php _e('Bars (5px)', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_mode_6" value="6" <?php echo $waveplayer_options['wave_mode'] == '6' ? 'selected' : '';?>><?php _e('Bars (6px)', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_mode_7" value="7" <?php echo $waveplayer_options['wave_mode'] == '7' ? 'selected' : '';?>><?php _e('Bars (7px)', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_mode_8" value="8" <?php echo $waveplayer_options['wave_mode'] == '8' ? 'selected' : '';?>><?php _e('Bars (8px)', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_mode_9" value="9" <?php echo $waveplayer_options['wave_mode'] == '9' ? 'selected' : '';?>><?php _e('Bars (9px)', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_mode_10" value="10" <?php echo $waveplayer_options['wave_mode'] == '10' ? 'selected' : '';?>><?php _e('Bars (10px)', 'waveplayer'); ?></option>
                                </select>
                                <span id="waveplayer_gap_width_group" class="<?php echo $waveplayer_options['wave_mode'] == '0' ? 'wvpl-inactive' : '';?>">
                                    <span class="wvpl-hspacer"><?php _e('with a', 'waveplayer'); ?></span>
                                    <select name="waveplayer_gap_width">
                                        <option name="waveplayer_gap_width_0" value="0" <?php echo $waveplayer_options['gap_width'] == '0' ? 'selected' : '';?>>0px</option>
                                        <option name="waveplayer_gap_width_1" value="1" <?php echo $waveplayer_options['gap_width'] == '1' ? 'selected' : '';?>>1px</option>
                                        <option name="waveplayer_gap_width_2" value="2" <?php echo $waveplayer_options['gap_width'] == '2' ? 'selected' : '';?>>2px</option>
                                        <option name="waveplayer_gap_width_3" value="3" <?php echo $waveplayer_options['gap_width'] == '3' ? 'selected' : '';?>>3px</option>
                                        <option name="waveplayer_gap_width_4" value="4" <?php echo $waveplayer_options['gap_width'] == '4' ? 'selected' : '';?>>4px</option>
                                        <option name="waveplayer_gap_width_5" value="5" <?php echo $waveplayer_options['gap_width'] == '5' ? 'selected' : '';?>>5px</option>
                                        <option name="waveplayer_gap_width_6" value="6" <?php echo $waveplayer_options['gap_width'] == '6' ? 'selected' : '';?>>6px</option>
                                        <option name="waveplayer_gap_width_7" value="7" <?php echo $waveplayer_options['gap_width'] == '7' ? 'selected' : '';?>>7px</option>
                                        <option name="waveplayer_gap_width_8" value="8" <?php echo $waveplayer_options['gap_width'] == '8' ? 'selected' : '';?>>8px</option>
                                        <option name="waveplayer_gap_width_9" value="9" <?php echo $waveplayer_options['gap_width'] == '9' ? 'selected' : '';?>>9px</option>
                                        <option name="waveplayer_gap_width_10" value="10" <?php echo $waveplayer_options['gap_width'] == '10' ? 'selected' : '';?>>10px</option>
                                    </select>
                                    <span class="wvpl-hspacer"><?php _e('gap between bars', 'waveplayer'); ?></span>
                                </span>
                                <br />
                                <span class="description"><?php _e('Defines the visualization mode of the waveform.', 'waveplayer'); ?></span>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_wave_compression"><?php _e('Wave compression: ', 'waveplayer' ); ?></label>
							</th>
							<td>
                                <select name="waveplayer_wave_compression">
                                    <option name="waveplayer_wave_compression_linear" value="1" <?php echo $waveplayer_options['wave_compression'] == '1' ? 'selected' : '';?>><?php _e('None (linear)', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_compression_square" value="2" <?php echo $waveplayer_options['wave_compression'] == '2' ? 'selected' : '';?>><?php _e('Moderate (square)', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_compression_cubic" value="3" <?php echo $waveplayer_options['wave_compression'] == '3' ? 'selected' : '';?>><?php _e('High (cubic)', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_compression_4th" value="4" <?php echo $waveplayer_options['wave_compression'] == '4' ? 'selected' : '';?>><?php _e('Very high (4th order)', 'waveplayer'); ?></option>
                                    <option name="waveplayer_wave_compression_5th" value="5" <?php echo $waveplayer_options['wave_compression'] == '5' ? 'selected' : '';?>><?php _e('Extreme (5th order)', 'waveplayer'); ?></option>
                                </select>
                                <br />
                                <span class="description"><?php _e('Defines the compression of the wave. This option does not affect the file of the audio, but only the way its waveform gets displayed.<br />
                                A lower level of compression shows a more evident difference between low intensity and high intensity in the waveform.<br />
                                Conversely, a higher level of compression flattens those differences, showing a more uniform wave.', 'waveplayer'); ?></span>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_wave_asymmetry"><?php _e('Wave asymmetry: ', 'waveplayer' ); ?></label>
							</th>
							<td>
                                <select name="waveplayer_wave_asymmetry">
                                    <option name="waveplayer_wave_asymmetry_1" value="1" <?php echo $waveplayer_options['wave_asymmetry'] == '1' ? 'selected' : '';?>>1/2 + 1/2</option>
                                    <option name="waveplayer_wave_asymmetry_2" value="2" <?php echo $waveplayer_options['wave_asymmetry'] == '2' ? 'selected' : '';?>>2/3 + 1/3</option>
                                    <option name="waveplayer_wave_asymmetry_3" value="3" <?php echo $waveplayer_options['wave_asymmetry'] == '3' ? 'selected' : '';?>>3/4 + 1/4</option>
                                    <option name="waveplayer_wave_asymmetry_4" value="4" <?php echo $waveplayer_options['wave_asymmetry'] == '4' ? 'selected' : '';?>>4/5 + 1/5</option>
                                    <option name="waveplayer_wave_asymmetry_5" value="5" <?php echo $waveplayer_options['wave_asymmetry'] == '5' ? 'selected' : '';?>>5/6 + 1/6</option>
                                </select>
                                <br />
                                <span class="description"><?php _e('Defines the ratio between the height of the top half of the wave and the height of the bottom part of the waves.', 'waveplayer'); ?></span>
							</td>
						</tr>
					</tbody>
				</table>
				<p class="submit">
					<input class="button button-primary" type="submit" name="submit" value="<?php _e('Save settings', 'waveplayer' ) ?>" />
				</p>
            </div>
            
            
            <!-- HTML & CSS TAB -->
			<div id="waveplayer-customization" class="waveplayer-option-page" <?php echo ($current_tab != 'waveplayer-customization' ? 'style="display:none"' : '');  ?>>
				<h2><?php _e( 'HTML & CSS Customization', 'waveplayer' ); ?></h2>
				<table class="form-table">
					<tbody>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_template"><?php _e('Info bar template: ', 'waveplayer' ); ?></label>
							</th>
							<td>
								<textarea name="waveplayer_template" size="120" rows="6"><?php echo $waveplayer_options['template']; ?></textarea>
								<span class="description">
                                    <?php _e('Use this text area to customize the HTML code that displays the information of the track.', 'waveplayer'); ?>
                                </span><br/>
								<span class="description">
									<?php _e('As a placeholder for a specific metadata, you can use any ID3 tag present in the audio file, delimited by the <code>%</code> character. 
                                    Please bear in mind that the presence of every tag is not guaranteed for every single track you upload.', 'waveplayer'); ?>
                                </span><br/>
                                <span class="description">
                                    <?php _e('Here is a list of the most common ID3 tags and metadata you can use:', 'waveplayer'); ?>
                                    <code>%album%</code>, <code>%artist%</code>, <code>%bitrate%</code>, <code>%bitrate_mode%</code>, <code>%channelmode%</code>, <code>%channels%</code>, 
                                    <code>%compression_ratio%</code>, <code>%dataformat%</code>, <code>%encoder_options%</code>, <code>%file%</code>, <code>%fileformat%</code>, 
                                    <code>%filesize%</code>, <code>%genre%</code>, <code>%length%</code>, <code>%length_formatted%</code>, <code>%lossless%</code>, 
                                    <code>%mime_type%</code>, <code>%sample_rate%</code>, <code>%title%</code>, <code>%year%</code>.
								</span>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								<label for="waveplayer_custom_css"><?php _e('Custom CSS: ', 'waveplayer' ); ?></label>
							</th>
							<td>
								<textarea name="waveplayer_custom_css" size="120" rows="6"><?php echo $waveplayer_options['custom_css']; ?></textarea>
								<span class="description"><?php _e('Use this text area to customize the CSS stylesheet of the player.', 'waveplayer'); ?></span>
							</td>
						</tr>
					</tbody>
				</table>
				<p class="submit">
					<input class="button button-primary" type="submit" name="submit" value="<?php _e('Save settings', 'waveplayer' ) ?>" />
				</p>
			</div>
		</form>


        <!-- MAINTENANCE TAB -->
        <div id="waveplayer-maintenance" class="waveplayer-option-page" <?php echo ($current_tab != 'waveplayer-maintenance' ? 'style="display:none"' : '');  ?>>
            <h2><?php _e( 'Maintenance', 'waveplayer' ); ?></h2>
            <p>
                <?php _e('A peak file is saved by WavePlayer in the \'<strong>peaks</strong>\' subfolder of the WordPress upload folder, whenever an audio track is loaded for the first time.<br/>
                This allows WavePlayer to render the waveform of each audio file much faster the following times.', 'waveplayer'); ?>
            </p>
            <hr />
            <p class="description">
                <?php _e('If you delete an audio track that was previously used in WavePlayer, the peak file remains unused forever in the peak subfolder of the plugin.<br />
                Although peak files are very small in size (usually around 30 kB), it can be a waste of your hosting space if you happen to upload and delete audio attachments regularly.<br />
                It is recommended to delete orphan peak files every time you delete a massive amount of audio attachments from your website.', 'waveplayer'); ?>
            </p>
            <p>
                <input class="button button-primary wvpl_delete_peaks" type="submit" data-elements="orphan" value="<?php _e('Delete orphan peak files', 'waveplayer' ) ?>" />
            </p>
            <hr />
            <p class="description">
                <?php _e('If you want to make sure WavePlayer regenerates all the peak files, you have to delete all of them using this button.', 'waveplayer'); ?>
            </p>
            <p>
                <input class="button button-primary wvpl_delete_peaks" type="submit" data-elements="all" value="<?php _e('Delete all peak files', 'waveplayer' ) ?>" />
            </p>
        </div>
	</div>
<?php
}


/**
*
* Registers the 'waveplayer' shortcode
*
*/
function waveplayer_shortcode( $atts, $content = '' ) {
    global $waveplayer_options;
    
    $args = shortcode_atts( array(
        'ids'                   =>  '',
        'mp3'                   =>  '',
        'size'                  =>  $waveplayer_options['size'],
        'style'                 =>  $waveplayer_options['style'],
        'shape'                 =>  $waveplayer_options['shape'],
        'autoplay'              =>  $waveplayer_options['autoplay'],
        'repeat_all'            =>  $waveplayer_options['repeat_all'],
        'shuffle'               =>  $waveplayer_options['shuffle'],
        'wave_color'            =>  $waveplayer_options['wave_color'],
        'wave_color_2'          =>  $waveplayer_options['wave_color_2'],
        'progress_color'        =>  $waveplayer_options['progress_color'],
        'progress_color_2'      =>  $waveplayer_options['progress_color_2'],
        'cursor_color'          =>  $waveplayer_options['cursor_color'],
        'cursor_color_2'        =>  $waveplayer_options['cursor_color_2'],
        'cursor_width'          =>  $waveplayer_options['cursor_width'],
        'hover_opacity'         =>  $waveplayer_options['hover_opacity'],
        'wave_mode'             =>  $waveplayer_options['wave_mode'],
        'gap_width'             =>  $waveplayer_options['gap_width'],
        'wave_compression'      =>  $waveplayer_options['wave_compression'],
        'wave_asymmetry'        =>  $waveplayer_options['wave_asymmetry'],
    ), $atts );
    
    switch ($args['size']) {
        case 'large':
        case 'lg':
            $args['size'] = 'lg';
            break;
        case 'medium':
        case 'md':
            $args['size'] = 'md';
            break;
        case 'small':
        case 'sm':
            $args['size'] = 'sm';
            break;
        case 'xsmall':
        case 'xs':
            $args['size'] = 'xs';
            break;
    }
    
    $classes_array = array();
    $classes_array[] = 'wvpl-size-'.$args['size'];
    $classes_array[] = 'wvpl-style-'.$args['style'];
    $classes_array[] = 'wvpl-shape-'.$args['shape'];
    
    $extra_classes = implode(' ', $classes_array);
    
    if ($args['shuffle']) {
        $ids_array = array_map('trim', explode(',', $args['ids']));
        shuffle($ids_array);
        $args['ids'] = implode(',', $ids_array);
    }
    
    $data = array();
    foreach($args as $name => $value) {
        $data[] = 'data-'.$name.'="'.$value.'"';
    }
    
    
    $html = '
        <div class="waveplayer ' . $extra_classes . '" ' . implode(' ', $data) . '>
            <div class="wvpl-left-box">
                <div class="wvpl-poster"></div>
                <div class="wvpl-interface">
                    <div class="wvpl-volume-overlay"></div>
                    <div class="wvpl-player"><audio preload="metadata"></audio></div>
                    <div class="wvpl-button wvpl-info"><span class="wvpl-icon fa fa-info"></span></div>
                    <div class="wvpl-controls">
                        <div class="wvpl-button wvpl-prev wvpl-disabled"><span class="wvpl-icon fa fa-angle-left"></span></div>
                        <div class="wvpl-button wvpl-play wvpl-disabled"><span class="wvpl-icon fa fa-play"></span></div>
                        <div class="wvpl-button wvpl-next wvpl-disabled"><span class="wvpl-icon fa fa-angle-right"></span></div>
                    </div>
                    <div class="wvpl-button wvpl-volume"><span class="wvpl-icon fa fa-volume-up"></span></div>
                </div>
            </div>
            <div class="wvpl-right-box">
                <div class="wvpl-waveform"></div>
                <div class="wvpl-infobar">
                    <div class="wvpl-playing-info"><div class="wvpl-infoblock"></div></div>
                    <div class="wvpl-playing-extrainfo"></div>
                </div>
            </div>
        </div>
    ';

    return $html;
}
add_shortcode( 'waveplayer', 'waveplayer_shortcode' );


/**
*
* Overrides the default audio element using a single track 'waveplayer' shortcode
*
*/
function waveplayer_audio_shortcode_override( $html, $attr, $content, $instances ) 
{
    global $waveplayer_options;
    
    $id = get_the_ID();
    if (get_post_type($id) != 'attachment') $id = '';
    
    $atts = array();
    $args = shortcode_atts( array(
        'ids'                   =>  '',
        'mp3'                   =>  $attr['src'],
        'size'                  =>  $waveplayer_options['size'],
        'style'                 =>  $waveplayer_options['style'],
        'shape'                 =>  $waveplayer_options['shape'],
        'autoplay'              =>  $waveplayer_options['autoplay'],
        'repeat_all'            =>  $waveplayer_options['repeat_all'],
        'shuffle'               =>  $waveplayer_options['shuffle'],
        'wave_color'            =>  $waveplayer_options['wave_color'],
        'wave_color_2'          =>  $waveplayer_options['wave_color_2'],
        'progress_color'        =>  $waveplayer_options['progress_color'],
        'progress_color_2'      =>  $waveplayer_options['progress_color_2'],
        'cursor_color'          =>  $waveplayer_options['cursor_color'],
        'cursor_color_2'        =>  $waveplayer_options['cursor_color_2'],
        'cursor_width'          =>  $waveplayer_options['cursor_width'],
        'hover_opacity'         =>  $waveplayer_options['hover_opacity'],
        'wave_mode'             =>  $waveplayer_options['wave_mode'],
        'gap_width'             =>  $waveplayer_options['gap_width'],
        'wave_compression'      =>  $waveplayer_options['wave_compression'],
        'wave_asymmetry'        =>  $waveplayer_options['wave_asymmetry'],
    ), $atts );

    $html = waveplayer_shortcode($args);
    return $html;
};
add_filter( 'wp_audio_shortcode_override', 'waveplayer_audio_shortcode_override', 10, 4 );


/**
*
* Generates an array of tracks based on the audio attachment ids provided
*
* @param    $ids, the list of audio track IDs
*
*/
function waveplayer_create_playlist( $ids ) {
    
    $ids_array = array_map('trim', explode(',', $ids));
    
    $tracks_array = array();
    foreach ($ids_array as $id) {
        $track = array();
        $track = wp_get_attachment_metadata($id);
        $track['index'] = $id;
        $track['file'] = wp_get_attachment_url($id); //MUSIC_PATH . sprintf('%04d.mp3', $id);
        $poster = wp_get_attachment_thumb_url( get_post_thumbnail_id( $id ) );
        $track['poster'] = is_string($poster) ? $poster : plugins_url( '/img/music_file.png' , __FILE__ );
        $tracks_array[] = $track;
    }
    return $tracks_array;
    
}


/**
*
* Sends a JSON array of the audio tracks to waveplayer.js through the AJAX call
*
* $_POST['param'] contains the list of ids in the playlist
*
*/
function waveplayer_ajax_load_playlist() {
        
    $tracks_array = waveplayer_create_playlist( $_POST['param'] );
	wp_send_json_success( array( 'tracks' => $tracks_array ) );

}
add_action( 'wp_ajax_waveplayer_load_playlist', 'waveplayer_ajax_load_playlist' );
add_action( 'wp_ajax_nopriv_waveplayer_load_playlist', 'waveplayer_ajax_load_playlist' );


/**
*
* Delete peak files of an audio track and sends an array of the peaks to waveplayer.js through the AJAX call
*
* $_POST['mode'] contains the mode of the deletion:
*                   - if 'all', deletes all the files in the peak subfolder of the plugin
*                   - if 'orphan', deletes the peak files corresponding to missing attachments
*
*/
function waveplayer_ajax_delete_peaks() {
    
    $mode = $_POST['mode'];
    $dir = trailingslashit( PEAK_FOLDER );
    
    $num = 0;
    $files = array();
    $dir = opendir( PEAK_FOLDER );
    switch ($mode) {
        case 'orphan':
            while ( false !== ( $file = readdir($dir) ) ) {
                if ( 'peaks' == pathinfo( $file, PATHINFO_EXTENSION ) ) {
                    $id = pathinfo($file, PATHINFO_FILENAME);
                    if ( get_post_mime_type( $id ) !== 'audio/mpeg' ) {
                        $num++;
                        $files[] = $file;
                        unlink( $dir . $file);
                    }
                }
            }
            break;
        case 'all':
            while (false !== ( $file = readdir($dir) ) ) {
                if ( 'peaks' == pathinfo( $file, PATHINFO_EXTENSION ) ) {
                    $num++;
                    $files[] = $file;
                    unlink( $dir . $file);
                }
            }
            break;
        default:
    }
    
    if ($num) {
        $message = $num == 1 ? $num . ' ' . __( 'file was successfully deleted.', 'waveplayer' )
                             : $num . ' ' . __( 'files were successfully deleted.', 'waveplayer' );
        wp_send_json_success( array( 'message' => $message, 'files' => $files ) );
    } else {
        $message = __( 'No files needed to be deleted.', 'waveplayer' );
        wp_send_json_error( array( 'message' => $message ) );
    }
}
add_action( 'wp_ajax_waveplayer_delete_peaks', 'waveplayer_ajax_delete_peaks' );
add_action( 'wp_ajax_nopriv_waveplayer_delete_peaks', 'waveplayer_ajax_delete_peaks' );


/**
*
* Reads the peak file of an audio track 
* and sends an array of the peaks to waveplayer.js through the AJAX call
*
* $_POST['id'] contains the id of the audio track
*
*/
function waveplayer_ajax_read_peaks() {
    
    $file = $_POST['id'] == 'sample' ? dirname( __FILE__ ) . '/peaks/sample.peaks' : trailingslashit( PEAK_FOLDER ) . $_POST['id'] . '.peaks';
    
    if ( file_exists( $file ) ) {
        $peaks = file_get_contents( $file );
    
        $peak_array = array_map('floatval', explode(',', $peaks));
        
//        $comments = get_comments( array( 'post_id' => $id ) );
//        wp_send_json_success( array( 'peaks' => $peak_array, 'comments' => $comments ) );
        wp_send_json_success( array( 'peaks' => $peak_array ) );
    }
    
    wp_send_json_error();
}
add_action( 'wp_ajax_waveplayer_read_peaks', 'waveplayer_ajax_read_peaks' );
add_action( 'wp_ajax_nopriv_waveplayer_read_peaks', 'waveplayer_ajax_read_peaks' );


/**
*
* Writes the peak file of an audio track only if not already present in the peak subfolder
* and sends a message to waveplayer.js through the AJAX call
*
* $_POST['id'] contains the id of the audio track
*
* $result is 'new' if the peak file was not already present in the peak subfolder, 'old' otherwise
*
*/
function waveplayer_ajax_write_peaks() {
    
    $dir = trailingslashit( PEAK_FOLDER );    
    $id = $_POST['id'];
    
    if ( !file_exists( $dir . $id . '.peaks' ) ) {
        $peaks = $_POST['peaks'];
        file_put_contents( $dir . $id . '.peaks', $peaks );
        wp_send_json_success();
    }
    wp_send_json_error();
}
add_action( 'wp_ajax_waveplayer_write_peaks', 'waveplayer_ajax_write_peaks' );
add_action( 'wp_ajax_nopriv_waveplayer_write_peaks', 'waveplayer_ajax_write_peaks' );


/**
*
* Reads ID3 tags of an MP3 file and sends an array of tags to waveplayer.js through the AJAX call
*
* $_POST['mp3'] contains the URL to a local MP3 file
*
*/
function waveplayer_ajax_read_mp3_tags() {
    
    $mp3 = $_SERVER['DOCUMENT_ROOT'] . parse_url($_POST['mp3'], PHP_URL_PATH);
    $tags = wp_read_audio_metadata($mp3);
    
	wp_send_json_success( array( 'tags' => $tags ) );
}
add_action( 'wp_ajax_waveplayer_read_mp3_tags', 'waveplayer_ajax_read_mp3_tags' );
add_action( 'wp_ajax_nopriv_waveplayer_read_mp3_tags', 'waveplayer_ajax_read_mp3_tags' );


/**
*
* Reads the waveplayer options and sends it as an array to waveplayer.js through the AJAX call
*
* $_POST['mp3'] contains the URL to a local MP3 file
*
*/
function waveplayer_ajax_load_options() {
    global $waveplayer_options;
    
//    echo json_encode($waveplayer_options);
	wp_send_json_success( array( 'options' => $waveplayer_options ) );
}
add_action( 'wp_ajax_waveplayer_load_options', 'waveplayer_ajax_load_options' );
add_action( 'wp_ajax_nopriv_waveplayer_load_options', 'waveplayer_ajax_load_options' );


/**
*
* Adds WavePlayer styles to the MCE Editor for a Visual rendition of the player
*
*/
function wpview_waveplayer_sandbox_styles() {
 	$version = '?ver=' . WAVEPLAYER_VERSION;
    
    $fontawesome = plugins_url( '/css/font-awesome.min.css' , __FILE__ ) . $version;
 	$waveplayer = plugins_url( '/css/style.css' , __FILE__ ) . $version;

	return array( $fontawesome, $waveplayer );
}


/**
*
* Renders a waveplayer shortcode and sends an head and body of an iframe to waveplayer.js through the AJAX call
*
* (this is used to render a [waveplayer] shortcode in the MCE Editor)
*
*/
function waveplayer_ajax_parse_shortcode() {
	global $post, $wp_scripts, $waveplayer_nonce;

	if ( empty( $_POST['shortcode'] ) ) {
		wp_send_json_error();
	}

	$shortcode = wp_unslash( $_POST['shortcode'] );
    
	if ( ! empty( $_POST['post_ID'] ) ) {
		$post = get_post( (int) $_POST['post_ID'] );
	}

	if ( ! $post || ! current_user_can( 'edit_post', $post->ID ) ) {
		if ( 'embed' === $shortcode ) {
			wp_send_json_error();
		}
	} else {
		setup_postdata( $post );
	}
    
	$parsed = do_shortcode( $shortcode );

	if ( empty( $parsed ) ) {
		wp_send_json_error( array(
			'type' => 'no-items',
			'message' => __( 'No items found.' ),
		) );
	}

	$head = '';
	$styles = wpview_waveplayer_sandbox_styles();
    $admin_ajax_url = admin_url('admin-ajax.php');
    $scripts = '
        <script type="text/javascript">
            /* <![CDATA[ */
            var waveplayer_ajax = {"ajax_url":"' . $admin_ajax_url . '","nonce":"'.$waveplayer_nonce.'"};
            /* ]]> */
        </script>
    ';
    if (SCRIPT_DEBUG) {
        $scripts .= '<script type="text/javascript" src="' . plugins_url( '/js/wavesurfer/wavesurfer.js', __FILE__ ) . '?ver=' . WAVESURFER_VERSION . '"></script>';
        $scripts .= '<script type="text/javascript" src="' . plugins_url( '/js/wavesurfer/util.js', __FILE__ ) . '?ver=' . WAVESURFER_VERSION . '"></script>';
        $scripts .= '<script type="text/javascript" src="' . plugins_url( '/js/wavesurfer/drawer.js', __FILE__ ) . '?ver=' . WAVESURFER_VERSION . '"></script>';
        $scripts .= '<script type="text/javascript" src="' . plugins_url( '/js/wavesurfer/drawer.canvas.js', __FILE__ ) . '?ver=' . WAVESURFER_VERSION . '"></script>';
        $scripts .= '<script type="text/javascript" src="' . plugins_url( '/js/wavesurfer/webaudio.js', __FILE__ ) . '?ver=' . WAVESURFER_VERSION . '"></script>';
    } else {
        $scripts .= '<script type="text/javascript" src="' . plugins_url( '/js/wavesurfer/wavesurfer.wvpl.min.js', __FILE__ ) . '?ver=' . WAVESURFER_VERSION . '"></script>';
    }
    $scripts .= '<script type="text/javascript" src="' . plugins_url( '/js/waveplayer.js', __FILE__ ) . '?ver=' . WAVEPLAYER_VERSION . '"/></script>';
    
	foreach ( $styles as $style ) {
		$head .= '<link type="text/css" rel="stylesheet" href="' . $style . '">';
	}

	if ( ! empty( $wp_scripts ) ) {
		$wp_scripts->done = array();
	}

    ob_start();
    
    echo $parsed;
    wp_print_scripts('jquery');
    echo $scripts;

	wp_send_json_success( array(
		'head' => $head,
		'body' => ob_get_clean()
	) );
}
add_action( 'wp_ajax_parse-waveplayer-shortcode', 'waveplayer_ajax_parse_shortcode' );


/**
*
* Renders a waveplayer shortcode and sends an head and body of an iframe to waveplayer.js through the AJAX call
*
* (this is used to render an [audio] shortcode in the MCE Editor)
*
*/
function waveplayer_ajax_parse_single_shortcode() {
	global $post, $wp_scripts, $waveplayer_nonce;

	if ( empty( $_POST['shortcode'] ) ) {
		wp_send_json_error();
	}

	$shortcode = wp_unslash( $_POST['shortcode'] );
    
	if ( ! empty( $_POST['post_ID'] ) ) {
		$post = get_post( (int) $_POST['post_ID'] );
	}

	// the embed shortcode requires a post
	if ( ! $post || ! current_user_can( 'edit_post', $post->ID ) ) {
		if ( 'embed' === $shortcode ) {
			wp_send_json_error();
		}
	} else {
		setup_postdata( $post );
	}
    
	$parsed = do_shortcode( $shortcode );

	if ( empty( $parsed ) ) {
		wp_send_json_error( array(
			'type' => 'no-items',
			'message' => __( 'No items found.' ),
		) );
	}

	$head = '';
	$styles = wpview_waveplayer_sandbox_styles();
    $admin_ajax_url = admin_url('admin-ajax.php');
    $scripts = '
        <script type="text/javascript">
            /* <![CDATA[ */
            var waveplayer_ajax = {"ajax_url":"' . $admin_ajax_url . '","nonce":"'.$waveplayer_nonce.'"};
            /* ]]> */
        </script>
    ';
    if (SCRIPT_DEBUG) {
        $scripts .= '<script type="text/javascript" src="' . plugins_url( '/js/wavesurfer/wavesurfer.js', __FILE__ ) . '?ver=' . WAVESURFER_VERSION . '"></script>';
        $scripts .= '<script type="text/javascript" src="' . plugins_url( '/js/wavesurfer/util.js', __FILE__ ) . '?ver=' . WAVESURFER_VERSION . '"></script>';
        $scripts .= '<script type="text/javascript" src="' . plugins_url( '/js/wavesurfer/drawer.js', __FILE__ ) . '?ver=' . WAVESURFER_VERSION . '"></script>';
        $scripts .= '<script type="text/javascript" src="' . plugins_url( '/js/wavesurfer/drawer.canvas.js', __FILE__ ) . '?ver=' . WAVESURFER_VERSION . '"></script>';
        $scripts .= '<script type="text/javascript" src="' . plugins_url( '/js/wavesurfer/webaudio.js', __FILE__ ) . '?ver=' . WAVESURFER_VERSION . '"></script>';
    } else {
        $scripts .= '<script type="text/javascript" src="' . plugins_url( '/js/wavesurfer/wavesurfer.wvpl.min.js', __FILE__ ) . '?ver=' . WAVESURFER_VERSION . '"></script>';
    }
    $scripts .= '<script type="text/javascript" src="' . plugins_url( '/js/waveplayer.js', __FILE__ ) . '?ver=' . WAVEPLAYER_VERSION . '"/></script>';
    
	foreach ( $styles as $style ) {
		$head .= '<link type="text/css" rel="stylesheet" href="' . $style . '">';
	}

	if ( ! empty( $wp_scripts ) ) {
		$wp_scripts->done = array();
	}

    ob_start();
    
    echo $parsed;
    wp_print_scripts('jquery');
    echo $scripts;

	wp_send_json_success( array(
		'head' => $head,
		'body' => ob_get_clean()
	) );
}
add_action( 'wp_ajax_parse-waveplayer-single-shortcode', 'waveplayer_ajax_parse_single_shortcode' );
